# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import six
import sys
import click
import re
from suds.cache import NoCache
from suds.client import Client
from suds.transport.https import HttpAuthenticated
from six.moves import urllib
import requests
import base64
from lxml import etree

from gestionatr.input.messages import message
from gestionatr.input.messages import message_gas
from gestionatr.input.messages.message import except_f1
from gestionatr.output.messages.sw_p0 import ENVELOP_BY_DISTR
from gestionatr.output.messages.sw_a5_29 import ENVELOP_GAS_BY_DISTR
from gestionatr.exceptions import *

from gestionatr import __version__


VERSION_TEXT = u'ATR library version (gestionatr) : {0}'.format(__version__)


P0_TEMPLATE = """
        <MensajeSolicitudInformacionAlRegistroDePS xmlns="http://localhost/elegibilidad">
          <Cabecera>
            <CodigoREEEmpresaEmisora>{emisora}</CodigoREEEmpresaEmisora>
            <CodigoREEEmpresaDestino>{destino}</CodigoREEEmpresaDestino>
            <CodigoDelProceso>P0</CodigoDelProceso>
            <CodigoDePaso>01</CodigoDePaso>
            <CodigoDeSolicitud>{solicitud}</CodigoDeSolicitud>
            <SecuencialDeSolicitud>01</SecuencialDeSolicitud>
            <FechaSolicitud>{fecha_solicitud}</FechaSolicitud>
            <CUPS>{cups}</CUPS>
          </Cabecera>
        </MensajeSolicitudInformacionAlRegistroDePS>
"""

A29_TEMPLATE = """
<sctdapplication xmlns="http://localhost/sctd/A529">
    <cabecera>
        <codenvio>GML</codenvio>
        <empresaemisora>{emisora}</empresaemisora>
        <empresadestino>{destino}</empresadestino>
        <fechacomunicacion>{fecha_solicitud}</fechacomunicacion>
        <horacomunicacion>{hora_solicitud}</horacomunicacion>
        <codproceso>29</codproceso>
        <tipomensaje>A5</tipomensaje>
    </cabecera>
    <a529>
        <fechacreacion>{fecha_solicitud}</fechacreacion>
        <horacreacion>{hora_solicitud}</horacreacion>
        <cups>{cups}</cups>
        <historicoconsumo>N</historicoconsumo>
    </a529>
</sctdapplication>
"""

def get_gestionatr_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(VERSION_TEXT)
    ctx.exit()


@click.group()
@click.option("--version", "-v", is_flag=True, callback=get_gestionatr_version,
              expose_value=False, is_eager=True, help="ATR library version")
def atr():
    pass


@atr.command(name='test')
@click.option("--filename", "-f", help="path to XML filename", required=True)
@click.option("--sector", "-s", help="e (power) or g (gas)", default="e")
def test(filename, sector):
    with open(filename, 'rb') as xml_file:
        try:
            data = xml_file.read()
            if sector == 'e':
                m = message.Message(data)
            elif sector == 'g':
                m = message_gas.MessageGas(data)
            m.parse_xml()
            sys.stdout.write('Correct File\n')
        except except_f1 as e:
            error_txt = six.text_type(e.value).encode(errors='ignore')
            sys.stdout.write(
                'WARNING: Invalid File: {0}\n'.format(error_txt)
            )
        except Exception as e:
            error_txt = six.text_type(e).encode(errors='ignore')
            sys.stdout.write(
                'WARNING: Invalid File: {0}\n'.format(error_txt)
            )
        finally:
            sys.stdout.flush()


def request_atr_29(url, user, password, xml_str=None, params=None):
    import random
    import re
    from datetime import datetime
    if xml_str is None and params is None:
        raise ValueError("XML or params must be passed to request_A5")
    if xml_str is None and params:
        codi_receptor = params['destino']
        params['fecha_solicitud'] = datetime.now().strftime('%Y-%m-%d')
        params['hora_solicitud'] = datetime.now().strftime('%H:%M:%S')
        params['solicitud'] = 10**11 + int((random.random() * 10**11))
        xml_str = re.sub(r'\s+<', '<', A29_TEMPLATE)
        xml_str = re.sub(r'\s+$', '', xml_str)
        xml_str = re.sub(r'\n', '', xml_str).format(**params)
    else:
        codi_receptor = xml_str.split("<empresadestino>")[1].split("</empresadestino>")[0]

    base64string = base64.encodestring(str('%s:%s' % (user, password))).replace('\n', '')
    headers = {
        "Authorization": "Basic %s" % base64string,
        "content-type": 'text/xml; charset=utf-8',
    }

    # Clean XML
    xml_str = xml_str.strip()
    xml_str = re.sub(r'<\?.*\?>', '', xml_str)
    xml_str = xml_str.replace("""<sctdapplication xmlns="http://localhost/sctd/A529">""", "")
    xml_str = xml_str.replace("""</sctdapplication>""", "")
    xml_str = xml_str.replace("<", "<a529:")
    xml_str = xml_str.replace("<a529:/", "</a529:")

    # Fem sempre 2 intents: amb el que esta definit per aquella distri i si va malament amb una plantilla base que
    # sol funcionar en la majoria. Aixi si una distri no la tenim documentada es fa el intent amb les 2 plantilles
    # principals que tenim. La "altres" i la "reintent"

    distri_envelop = ENVELOP_GAS_BY_DISTR.get(codi_receptor, ENVELOP_GAS_BY_DISTR.get("altres"))
    error = None
    for envelop in [distri_envelop]:
        xml_str_to_use = xml_str
        soap_content = envelop['template'].format(xml_str=xml_str_to_use)

        # Send request
        h = headers.copy()
        h.update(envelop['extra_headers'])
        res = requests.post(url, data=soap_content, headers=h, auth=(user, password), verify=False)
        res = res.content
        try:
            def find_child(element, child_name):
                res = None
                if child_name in element.tag:
                    return element
                for child in element:
                    res = find_child(child, child_name)
                    if res is not None:
                        break
                return res

            res = re.sub(r'\<[^: \n>/]+:', '<', res)
            res = re.sub(r'\</[^: \n>/]+:', '</', res)
            res = re.sub(r'<\?.*\?>', '', res)
            res = re.sub(r'<cabecera.*?>', '<cabecera>', res)
            res = res.replace("<consultaCupsResponse>", """<sctdapplication>""")
            res = res.replace("</consultaCupsResponse>", """</sctdapplication>""", 1)
            aux = etree.fromstring(res)
            aux_res = find_child(aux, "sctdapplication")

            res = etree.tostring(aux_res)
            return res
        except A529FaultError as A529efe:
            if not error:
                error = A529efe
        except Exception as e:
            if not error:
                error = e

    if error:
        if isinstance(error, A529FaultError):
            raise error
        else:
            print(error)

    return res


def request_p0(url, user, password, xml_str=None, params=None):
    import random
    import re
    from datetime import datetime
    if xml_str is None and params is None:
        raise ValueError("XML or params must be passed to request_p0")
    if xml_str is None and params:
        codi_receptor = params['destino']
        params['fecha_solicitud'] = datetime.now().strftime('%Y-%m-%dT%H:%M:%S')
        params['solicitud'] = 10**11 + int((random.random() * 10**11))
        xml_str = re.sub(r'\s+<', '<', P0_TEMPLATE)
        xml_str = re.sub(r'\s+$', '', xml_str)
        xml_str = re.sub(r'\n', '', xml_str).format(**params)
    else:
        codi_receptor = xml_str.split("<CodigoREEEmpresaDestino>")[1].split("</CodigoREEEmpresaDestino>")[0]

    base64string = base64.encodestring('%s:%s' % (user, password)).replace('\n', '')
    headers = {
        "Authorization": "Basic %s" % base64string,
        "content-type": 'text/xml; charset=utf-8',
    }

    # Clean XML
    xml_str = xml_str.strip()
    xml_str = xml_str.replace("'utf-8'", "'UTF-8'")
    xml_str = xml_str.replace("<?xml version='1.0' encoding='UTF-8'?>", "")

    # Fem sempre 2 intents: amb el que esta definit per aquella distri i si va malament amb una plantilla base que
    # sol funcionar en la majoria. Aixi si una distri no la tenim documentada es fa el intent amb les 2 plantilles
    # principals que tenim. La "altres" i la "reintent"
    distri_envelop = ENVELOP_BY_DISTR.get(codi_receptor, ENVELOP_BY_DISTR.get("altres"))
    retry_envelop = ENVELOP_BY_DISTR.get("reintent")
    retry2_envelop = ENVELOP_BY_DISTR.get("reintent_2")
    error = None
    for envelop in [distri_envelop, retry_envelop, retry2_envelop]:
        xml_str_to_use = xml_str
        soap_content = envelop['template'].format(xml_str=xml_str_to_use)

        # Send request
        h = headers.copy()
        h.update(envelop['extra_headers'])
        res = requests.post(url, data=soap_content, headers=h, auth=(user, password), verify=False)
        res = res.content
        try:
            def find_child(element, child_name):
                res = None
                if child_name in element.tag:
                    return element
                for child in element:
                    res = find_child(child, child_name)
                    if res is not None:
                        break
                return res

            aux = etree.fromstring(res)
            aux_res = find_child(aux, "MensajeEnvioInformacionPS")
            if aux_res is None:
                aux_res = find_child(aux, "MensajeRechazoP0")
            if aux_res is None:
                aux_res = find_child(aux, "faultstring")
                if aux_res is not None:
                    error_mssg = {
                        'request': res,
                        'response': etree.tostring(aux_res)
                    }
                    raise P0FaultError(error_mssg)

            res = etree.tostring(aux_res)
            return res
        except P0FaultError as p0efe:
            if not error:
                error = p0efe
        except Exception as e:
            if not error:
                error = e

    if error:
        if isinstance(error, P0FaultError):
            raise error
        else:
            print(error)

    return res


@atr.command(name='p0')
@click.option('-u', '--url', default='http://localhost', help=u'URL del webservice', show_default=True)
@click.option('-s', '--user', default='admin', help=u'User del webservice', show_default=True)
@click.option('-p', '--password', default='admin', help=u'Password del webservice', show_default=True)
@click.option('-f', '--file', help=u'Fitxer P0 pas 01 per enviar', show_default=True)
@click.option('--emisora', help='Código REE empresa emisora')
@click.option('--destino', help='Código REE empresa destino')
@click.option('--cups', help='CUPS')
def sollicitar_p0(url, user, password, file=None, emisora=None, destino=None, cups=None):
    params = None
    from lxml import etree
    if emisora and destino and cups:
        params = {
            'emisora': emisora,
            'destino': destino,
            'cups': cups
        }

    res = request_p0(url, user, password, file, params)
    res = etree.fromstring(res)
    print(etree.tostring(res, pretty_print=True))

@atr.command(name='a529')
@click.option('-u', '--url', default='http://localhost', help=u'URL del webservice', show_default=True)
@click.option('-s', '--user', default='admin', help=u'User del webservice', show_default=True)
@click.option('-p', '--password', default='admin', help=u'Password del webservice', show_default=True)
@click.option('-f', '--file', help=u'Fitxer 29 pas A5 per enviar', show_default=True)
@click.option('--emisora', help='Código REE empresa emisora')
@click.option('--destino', help='Código REE empresa destino')
@click.option('--cups', help='CUPS')
def sollicitar_a529(url, user, password, file=None, emisora=None, destino=None, cups=None):
    params = None
    from lxml import etree
    if emisora and destino and cups:
        params = {
            'emisora': emisora,
            'destino': destino,
            'cups': cups
        }

    res = request_atr_29(url, user, password, file, params)
    res = etree.fromstring(res)
    print(etree.tostring(res, pretty_print=True))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from .message import Message
from .Deadlines import ProcessDeadline, DeadLine, Workdays, Naturaldays
from gestionatr.utils import get_rec_attr


class C1(Message, ProcessDeadline):
    """Clase que implementa C1."""

    steps = [
        DeadLine('01', Workdays(5)),
        DeadLine('02', Workdays(1)),
        DeadLine('05', Workdays(1)),
        DeadLine('06', Workdays(1)),
        DeadLine('08', Workdays(5)),
    ]

    # Datos paso 01
    @property
    def datos_solicitud(self):
        tree = '{0}.DatosSolicitud'.format(self._header)
        sol = get_rec_attr(self.obj, tree, False)
        if sol not in [None, False]:
            return DatosSolicitud(sol)
        else:
            return False

    @property
    def cliente(self):
        tree = '{0}.Cliente'.format(self._header)
        cli = get_rec_attr(self.obj, tree, False)
        if cli not in [None, False]:
            return Cliente(cli)
        else:
            return False

    @property
    def comentarios(self):
        tree = '{0}.Comentarios'.format(self._header)
        com = get_rec_attr(self.obj, tree, False)
        if com:
            return com.text
        else:
            return False

    @property
    def registros_documento(self):
        data = []
        obj = get_rec_attr(self.obj, self._header, False)
        if not hasattr(obj, 'RegistrosDocumento'):
            obj = get_rec_attr(self.obj, 'Rechazos', False)
        if (hasattr(obj, 'RegistrosDocumento') and
                hasattr(obj.RegistrosDocumento, 'RegistroDoc')):
            for d in obj.RegistrosDocumento.RegistroDoc:
                data.append(RegistroDoc(d))
        return data

    # Datos paso 02 aceptacion
    @property
    def datos_aceptacion(self):
        tree = '{0}.DatosAceptacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DatosAceptacion(data)
        else:
            return False

    @property
    def contrato(self):
        tree = '{0}.Contrato'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return Contrato(data)
        else:
            return False

    # Datos paso 02 rechazo y paso 04
    @property
    def rechazos(self):
        obj = getattr(self.obj, 'Rechazos')
        data = []
        if obj not in [None, False]:
            for i in obj.Rechazo:
                data.append(Rechazo(i))
            return data
        return data

    @property
    def fecha_rechazo(self):
        tree = '{0}.FechaRechazo'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data:
            return data.text
        else:
            return False

    # Datos paso 05
    @property
    def datos_activacion(self):
        tree = '{0}.DatosActivacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DatosActivacion(data)
        else:
            return False

    @property
    def puntos_medida(self):
        data = []
        obj = get_rec_attr(self.obj, self._header, False)
        if (hasattr(obj, 'PuntosDeMedida') and
                hasattr(obj.PuntosDeMedida, 'PuntoDeMedida')):
            for d in obj.PuntosDeMedida.PuntoDeMedida:
                data.append(PuntoDeMedida(d))
        return data

    # Datos paso 06
    @property
    def datos_notificacion(self):
        tree = '{0}.DatosNotificacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DatosNotificacion(data)
        else:
            return False

    # Datos paso 09 i 10
    @property
    def fecha_aceptacion(self):
        tree = '{0}.FechaAceptacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data:
            return data.text
        else:
            return False

    # Datos paso 11
    @property
    def ind_bono_social(self):
        tree = '{0}.IndBonoSocial'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data:
            return data.text
        else:
                return False

    @property
    def fecha_activacion_prevista(self):
        tree = '{0}.FechaActivacionPrevista'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data:
            return data.text
        else:
            return False


class DatosSolicitud(object):

    def __init__(self, data):
        self.datos_solicitud = data

    @property
    def ind_activacion(self):
        data = ''
        try:
            data = self.datos_solicitud.IndActivacion.text
        except AttributeError:
            pass
        return data

    @property
    def ind_esencial(self):
        data = False
        try:
            data = self.datos_solicitud.IndEsencial.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_ultimo_movimiento_ind_esencial(self):
        data = False
        try:
            data = self.datos_solicitud.FechaUltimoMovimientoIndEsencial.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_prevista_accion(self):
        data = False
        try:
            data = self.datos_solicitud.FechaPrevistaAccion.text
        except AttributeError:
            pass
        return data

    @property
    def contratacion_incondicional_ps(self):
        data = ''
        try:
            data = self.datos_solicitud.ContratacionIncondicionalPS.text
        except AttributeError:
            pass
        return data

    @property
    def contratacion_incondicional_bs(self):
        data = ''
        try:
            data = self.datos_solicitud.ContratacionIncondicionalBS.text
        except AttributeError:
            pass
        return data

    @property
    def bono_social(self):
        data = ''
        try:
            data = self.datos_solicitud.BonoSocial.text
        except AttributeError:
            pass
        return data


class Cliente(object):

    def __init__(self, data):
        self.cliente = data

    @property
    def tipo_identificador(self):
        data = ''
        try:
            data = self.cliente.IdCliente.TipoIdentificador.text
        except AttributeError:
            pass
        return data

    @property
    def identificador(self):
        data = ''
        try:
            data = self.cliente.IdCliente.Identificador.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_persona(self):
        data = ''
        try:
            data = self.cliente.IdCliente.TipoPersona.text
        except AttributeError:
            try:
                nom = self.cliente.Nombre.NombreDePila.text
                data = 'F'
            except AttributeError:
                try:
                    nom = self.cliente.Nombre.RazonSocial.text
                    data = 'J'
                except AttributeError:
                    pass
        return data

    @property
    def nombre_de_pila(self):
        data = ''
        try:
            data = self.cliente.Nombre.NombreDePila.text
        except AttributeError:
            pass
        return data

    @property
    def primer_apellido(self):
        data = ''
        try:
            data = self.cliente.Nombre.PrimerApellido.text
        except AttributeError:
            pass
        return data

    @property
    def segundo_apellido(self):
        data = ''
        try:
            data = self.cliente.Nombre.SegundoApellido.text
        except AttributeError:
            pass
        return data

    @property
    def razon_social(self):
        data = ''
        try:
            data = self.cliente.Nombre.RazonSocial.text
        except AttributeError:
            pass
        return data

    @property
    def nombre(self):
        """Nombre completo según sea persona física o jurídica"""
        if self.tipo_persona == 'F':
            nom = '{0}, {1}'.format(
                (' '.join([self.primer_apellido, self.segundo_apellido])).strip(),
                self.nombre_de_pila
            )
        else:
            nom = self.razon_social
        return nom

    @property
    def telefonos(self):
        data = []
        try:
            for tel in self.cliente.Telefono:
                data.append((tel.PrefijoPais.text, tel.Numero.text))
        except AttributeError:
            pass
        return data

    @property
    def correo_electronico(self):
        data = ''
        try:
            data = self.cliente.CorreoElectronico.text
        except AttributeError:
            pass
        return data


class RegistroDoc(object):

    def __init__(self, data):
        self.doc = data

    @property
    def tipo_doc_aportado(self):
        data = ''
        try:
            data = self.doc.TipoDocAportado.text
        except AttributeError:
            pass
        return data

    @property
    def direccion_url(self):
        data = ''
        try:
            data = self.doc.DireccionUrl.text
        except AttributeError:
            pass
        return data


class DatosAceptacion(object):

    def __init__(self, data):
        self.datos_aceptacion = data

    @property
    def fecha_aceptacion(self):
        data = False
        try:
            data = self.datos_aceptacion.FechaAceptacion.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_ultima_lectura_firme(self):
        data = False
        try:
            data = self.datos_aceptacion.FechaUltimaLecturaFirme.text
        except AttributeError:
            pass
        return data

    @property
    def potencia_actual(self):
        data = ''
        try:
            data = self.datos_aceptacion.PotenciaActual.text
        except AttributeError:
            pass
        return data

    @property
    def actuacion_campo(self):
        data = ''
        try:
            data = self.datos_aceptacion.ActuacionCampo.text
        except AttributeError:
            pass
        return data

    @property
    def bono_social(self):
        data = ''
        try:
            data = self.datos_aceptacion.BonoSocial.text
        except AttributeError:
            pass
        return data


class Contrato(object):

    def __init__(self, data):
        self.contrato = data

    @property
    def tipo_contrato_atr(self):
        data = ''
        try:
            data = self.contrato.TipoContratoATR.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_activacion_prevista(self):
        data = ''
        try:
            data = self.contrato.TipoActivacionPrevista.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_activacion_prevista(self):
        data = False
        try:
            data = self.contrato.FechaActivacionPrevista.text
        except AttributeError:
            pass
        return data

    @property
    def tarifa_atr(self):
        data = ''
        try:
            data = self.contrato.CondicionesContractuales.TarifaATR.text
        except AttributeError:
            pass
        return data

    @property
    def modo_control_potencia(self):
        data = ''
        try:
            data = self.contrato.CondicionesContractuales.ModoControlPotencia.text
        except AttributeError:
            pass
        return data

    @property
    def potencias_contratadas(self):
        data = []
        obj = self.contrato.CondicionesContractuales
        if (hasattr(obj, 'PotenciasContratadas') and
                hasattr(obj.PotenciasContratadas, 'Potencia')):
            for d in obj.PotenciasContratadas.Potencia:
                data.append((int(d.get('Periodo')), int(d.text)))
        return data

    @property
    def cod_contrato(self):
        data = ''
        try:
            data = self.contrato.IdContrato.CodContrato.text
        except AttributeError:
            pass
        return data

    @property
    def data_finalitzacio(self):
        data = ''
        try:
            data = self.contrato.FechaFinalizacion.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_finalizacion(self):
        data = ''
        try:
            data = self.contrato.FechaFinalizacion.text
        except AttributeError:
            pass
        return data

    @property
    def autoconsumo(self):
        data = ''
        try:
            data = Autoconsumo(self.contrato.Autoconsumo)
        except AttributeError:
            pass
        return data

    @property
    def periodicidad_facturacion(self):
        data = ''
        try:
            data = self.contrato.CondicionesContractuales.PeriodicidadFacturacion.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_de_telegestion(self):
        data = ''
        try:
            data = self.contrato.CondicionesContractuales.TipodeTelegestion.text
        except AttributeError:
            pass
        return data

    @property
    def marca_medida_con_perdidas(self):
        data = ''
        try:
            data = self.contrato.CondicionesContractuales.MarcaMedidaConPerdidas.text
        except AttributeError:
            pass
        return data

    @property
    def tension_del_suministro(self):
        data = ''
        try:
            data = self.contrato.CondicionesContractuales.TensionDelSuministro.text
        except AttributeError:
            pass
        return data

    @property
    def vas_trafo(self):
        data = ''
        try:
            data = self.contrato.CondicionesContractuales.VAsTrafo.text
        except AttributeError:
            pass
        return data

    @property
    def porcentaje_perdidas(self):
        data = ''
        try:
            data = self.contrato.CondicionesContractuales.PorcentajePerdidas.text
        except AttributeError:
            pass
        return data


class Rechazo(object):

    def __init__(self, data):
        self.rechazo = data

    @property
    def secuencial(self):
        data = ''
        try:
            data = self.rechazo.Secuencial.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_motivo(self):
        data = ''
        try:
            data = self.rechazo.CodigoMotivo.text
        except AttributeError:
            pass
        return data

    @property
    def comentarios(self):
        data = ''
        try:
            data = self.rechazo.Comentarios.text
        except AttributeError:
            pass
        return data


class DatosActivacion(object):

    def __init__(self, data):
        self.datos_activacion = data

    @property
    def fecha(self):
        data = False
        try:
            data = self.datos_activacion.Fecha.text
        except AttributeError:
            pass
        return data

    @property
    def bono_social(self):
        data = ''
        try:
            data = self.datos_activacion.BonoSocial.text
        except AttributeError:
            pass
        return data

    @property
    def ind_esencial(self):
        data = False
        try:
            data = self.datos_activacion.IndEsencial.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_ultimo_movimiento_ind_esencial(self):
        data = False
        try:
            data = self.datos_activacion.FechaUltimoMovimientoIndEsencial.text
        except AttributeError:
            pass
        return data


class PuntoDeMedida(object):

    def __init__(self, data):
        self.pm = data

    @property
    def cod_pm(self):
        data = ''
        try:
            data = self.pm.CodPM.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_movimiento(self):
        data = ''
        try:
            data = self.pm.TipoMovimiento.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_pm(self):
        data = ''
        try:
            data = self.pm.TipoPM.text
        except AttributeError:
            pass
        return data

    @property
    def cod_pm_principal(self):
        data = ''
        try:
            data = self.pm.CodPMPrincipal.text
        except AttributeError:
            pass
        return data

    @property
    def modo_lectura(self):
        data = ''
        try:
            data = self.pm.ModoLectura.text
        except AttributeError:
            pass
        return data

    @property
    def funcion(self):
        data = ''
        try:
            data = self.pm.Funcion.text
        except AttributeError:
            pass
        return data

    @property
    def direccion_enlace(self):
        data = ''
        try:
            data = self.pm.DireccionEnlace.text
        except AttributeError:
            pass
        return data

    @property
    def direccion_punto_medida(self):
        data = ''
        try:
            data = self.pm.DireccionPuntoMedida.text
        except AttributeError:
            pass
        return data

    @property
    def num_linea(self):
        data = ''
        try:
            data = self.pm.NumLinea.text
        except AttributeError:
            pass
        return data

    @property
    def telefono_telemedida(self):
        data = ''
        try:
            data = self.pm.TelefonoTelemedida.text
        except AttributeError:
            pass
        return data

    @property
    def estado_telefono(self):
        data = ''
        try:
            data = self.pm.EstadoTelefono.text
        except AttributeError:
            pass
        return data

    @property
    def clave_acceso(self):
        data = ''
        try:
            data = self.pm.ClaveAcceso.text
        except AttributeError:
            pass
        return data

    @property
    def tension_pm(self):
        data = ''
        try:
            data = self.pm.TensionPM.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_vigor(self):
        data = False
        try:
            data = self.pm.FechaVigor.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_alta(self):
        data = False
        try:
            data = self.pm.FechaAlta.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_baja(self):
        data = False
        try:
            data = self.pm.FechaBaja.text
        except AttributeError:
            pass
        return data

    @property
    def comentarios(self):
        data = ''
        try:
            data = self.pm.Comentarios.text
        except AttributeError:
            pass
        return data

    @property
    def aparatos(self):
        data = []
        obj = get_rec_attr(self.pm, "Aparatos", False)
        if obj is not None and hasattr(obj, 'Aparato'):
            for d in obj.Aparato:
                data.append(Aparato(d))
        return data


class Aparato(object):

    def __init__(self, data):
        self.aparato = data

    @property
    def tipo_aparato(self):
        data = ''
        try:
            data = self.aparato.ModeloAparato.TipoAparato.text
        except AttributeError:
            pass
        return data

    @property
    def marca_aparato(self):
        data = ''
        try:
            data = self.aparato.ModeloAparato.MarcaAparato.text
        except AttributeError:
            pass
        return data

    @property
    def modelo_marca(self):
        data = ''
        try:
            data = self.aparato.ModeloAparato.ModeloMarca.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_movimiento(self):
        data = ''
        try:
            data = self.aparato.TipoMovimiento.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_equipo_medida(self):
        data = ''
        try:
            data = self.aparato.TipoEquipoMedida.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_propiedad_aparato(self):
        data = ''
        try:
            data = self.aparato.TipoPropiedadAparato.text
        except AttributeError:
            pass
        return data

    @property
    def propietario(self):
        data = ''
        try:
            data = self.aparato.Propietario.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_dhedm(self):
        data = ''
        try:
            data = self.aparato.TipoDHEdM.text
        except AttributeError:
            pass
        return data

    @property
    def modo_medida_potencia(self):
        data = ''
        try:
            data = self.aparato.ModoMedidaPotencia.text
        except AttributeError:
            pass
        return data

    @property
    def lectura_directa(self):
        data = ''
        try:
            data = self.aparato.LecturaDirecta.text
        except AttributeError:
            pass
        return data

    @property
    def cod_precinto(self):
        data = ''
        try:
            data = self.aparato.CodPrecinto.text
        except AttributeError:
            pass
        return data

    @property
    def periodo_fabricacion(self):
        data = ''
        try:
            data = self.aparato.DatosAparato.PeriodoFabricacion.text
        except AttributeError:
            pass
        return data

    @property
    def numero_serie(self):
        data = ''
        try:
            data = self.aparato.DatosAparato.NumeroSerie.text
        except AttributeError:
            pass
        return data

    @property
    def funcion_aparato(self):
        data = ''
        try:
            data = self.aparato.DatosAparato.FuncionAparato.text
        except AttributeError:
            pass
        return data

    @property
    def num_integradores(self):
        data = ''
        try:
            data = self.aparato.DatosAparato.NumIntegradores.text
        except AttributeError:
            pass
        return data

    @property
    def constante_energia(self):
        data = ''
        try:
            data = self.aparato.DatosAparato.ConstanteEnergia.text
        except AttributeError:
            pass
        return data

    @property
    def constante_maximetro(self):
        data = ''
        try:
            data = self.aparato.DatosAparato.ConstanteMaximetro.text
        except AttributeError:
            pass
        return data

    @property
    def ruedas_enteras(self):
        data = ''
        try:
            data = self.aparato.DatosAparato.RuedasEnteras.text
        except AttributeError:
            pass
        return data

    @property
    def ruedas_decimales(self):
        data = ''
        try:
            data = self.aparato.DatosAparato.RuedasDecimales.text
        except AttributeError:
            pass
        return data

    @property
    def medidas(self):
        data = []
        obj = get_rec_attr(self.aparato, "Medidas", False)
        if obj is not None and hasattr(obj, 'Medida'):
            for d in obj.Medida:
                data.append(Medida(d))
        return data


class Medida(object):

    def __init__(self, data):
        self.medida = data

    @property
    def tipo_dhedm(self):
        data = ''
        try:
            data = self.medida.TipoDHEdM.text
        except AttributeError:
            pass
        return data

    @property
    def periodo(self):
        data = ''
        try:
            data = self.medida.Periodo.text
        except AttributeError:
            pass
        return data

    @property
    def magnitud_medida(self):
        data = ''
        try:
            data = self.medida.MagnitudMedida.text
        except AttributeError:
            pass
        return data

    @property
    def procedencia(self):
        data = ''
        try:
            data = self.medida.Procedencia.text
        except AttributeError:
            pass
        return data

    @property
    def ultima_lectura_firme(self):
        data = ''
        try:
            data = self.medida.UltimaLecturaFirme.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_lectura_firme(self):
        data = False
        try:
            data = self.medida.FechaLecturaFirme.text
        except AttributeError:
            pass
        return data

    @property
    def anomalia(self):
        data = ''
        try:
            data = self.medida.Anomalia.text
        except AttributeError:
            pass
        return data

    @property
    def comentarios(self):
        data = ''
        try:
            data = self.medida.Comentarios.text
        except AttributeError:
            pass
        return data


class DatosNotificacion(object):

    def __init__(self, data):
        self.datos_notificacion = data

    @property
    def fecha_activacion(self):
        data = False
        try:
            data = self.datos_notificacion.FechaActivacion.text
        except AttributeError:
            pass
        return data

    @property
    def ind_bono_social(self):
        data = False
        try:
            data = self.datos_notificacion.IndBonoSocial.text
        except AttributeError:
            pass
        return data


class Autoconsumo(object):

    def __init__(self, data):
        self.autoconsumo = data

    @property
    def datos_suministro(self):
        data = ''
        try:
            data = DatosSuministro(self.autoconsumo.DatosSuministro)
        except AttributeError:
            pass
        return data

    @property
    def datos_cau(self):
        data = []
        try:
            for datos in self.autoconsumo.DatosCAU:
                data.append(DatosCAU(datos))
        except AttributeError:
            pass
        return data


class DatosSuministro(object):

    def __init__(self, data):
        self.datos_suministro = data

    @property
    def tipo_cups(self):
        data = ''
        try:
            data = self.datos_suministro.TipoCUPS.text
        except AttributeError:
            pass
        return data

    @property
    def ref_catastro(self):
        data = ''
        try:
            data = self.datos_suministro.RefCatastro.text
        except AttributeError:
            pass
        return data


class DatosCAU(object):

    def __init__(self, data):
        self.datos_cau = data

    @property
    def cau(self):
        data = ''
        try:
            data = self.datos_cau.CAU.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_autoconsumo(self):
        data = ''
        try:
            data = self.datos_cau.TipoAutoconsumo.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_subseccion(self):
        data = ''
        try:
            data = self.datos_cau.TipoSubseccion.text
        except AttributeError:
            pass
        return data

    @property
    def colectivo(self):
        data = ''
        try:
            data = self.datos_cau.Colectivo.text
        except AttributeError:
            pass
        return data

    @property
    def datos_inst_gen(self):
        data = []
        try:
            for datos in self.datos_cau.DatosInstGen:
                data.append(DatosInstGen(datos))
        except AttributeError:
            pass
        return data


class DatosInstGen(object):

    def __init__(self, data):
        self.datos_inst_gen = data

    @property
    def cil(self):
        data = ''
        try:
            data = self.datos_inst_gen.CIL.text
        except AttributeError:
            pass
        return data

    @property
    def tec_generador(self):
        data = ''
        try:
            data = self.datos_inst_gen.TecGenerador.text
        except AttributeError:
            pass
        return data

    @property
    def combustible(self):
        data = ''
        try:
            data = self.datos_inst_gen.Combustible.text
        except AttributeError:
            pass
        return data

    @property
    def pot_instalada_gen(self):
        data = ''
        try:
            data = self.datos_inst_gen.PotInstaladaGen.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_instalacion(self):
        data = ''
        try:
            data = self.datos_inst_gen.TipoInstalacion.text
        except AttributeError:
            pass
        return data

    @property
    def esquema_medida(self):
        data = ''
        try:
            data = self.datos_inst_gen.EsquemaMedida.text
        except AttributeError:
            pass
        return data

    @property
    def ssaa(self):
        data = ''
        try:
            data = self.datos_inst_gen.SSAA.text
        except AttributeError:
            pass
        return data

    @property
    def unico_contrato(self):
        data = ''
        try:
            data = self.datos_inst_gen.UnicoContrato.text
        except AttributeError:
            pass
        return data

    @property
    def ref_catastro(self):
        data = ''
        try:
            data = self.datos_inst_gen.RefCatastro.text
        except AttributeError:
            pass
        return data

    @property
    def utm(self):
        data = ''
        try:
            data = UTM(self.datos_inst_gen.UTM)
        except AttributeError:
            pass
        return data

    @property
    def titular_representante_gen(self):
        data = ''
        try:
            data = TitularRepresentanteGen(self.datos_inst_gen.TitularRepresentanteGen)
        except AttributeError:
            pass
        return data


class UTM(object):

    def __init__(self, data):
        self.utm = data

    @property
    def x(self):
        data = ''
        try:
            data = self.utm.X.text
        except AttributeError:
            pass
        return data

    @property
    def y(self):
        data = ''
        try:
            data = self.utm.Y.text
        except AttributeError:
            pass
        return data

    @property
    def huso(self):
        data = ''
        try:
            data = self.utm.Huso.text
        except AttributeError:
            pass
        return data

    @property
    def banda(self):
        data = ''
        try:
            data = self.utm.Banda.text
        except AttributeError:
            pass
        return data


class TitularRepresentanteGen(object):

    def __init__(self, data):
        self.titular_representante_gen = data

    @property
    def id_titular(self):
        data = ''
        try:
            data = IdTitular(self.titular_representante_gen.IdTitular)
        except AttributeError:
            pass
        return data

    @property
    def nombre(self):
        data = ''
        try:
            data = Nombre(self.titular_representante_gen.Nombre)
        except AttributeError:
            pass
        return data

    @property
    def telefono(self):
        data = []
        try:
            for telefono in self.titular_representante_gen.Telefono:
                data.append((telefono.PrefijoPais.text, telefono.Numero.text))
        except AttributeError:
            pass
        return data

    @property
    def correo_electronico(self):
        data = ''
        try:
            data = self.titular_representante_gen.CorreoElectronico.text
        except AttributeError:
            pass
        return data


class IdTitular(object):

    def __init__(self, data):
        self.id_titular = data

    @property
    def tipo_identificador(self):
        data = ''
        try:
            data = self.id_titular.TipoIdentificador.text
        except AttributeError:
            pass
        return data

    @property
    def identificador(self):
        data = ''
        try:
            data = self.id_titular.Identificador.text
        except AttributeError:
            pass
        return data


class Nombre(object):

    def __init__(self, data):
        self.nombre = data

    @property
    def nombre_de_pila(self):
        data = ''
        try:
            data = self.nombre.NombreDePila.text
        except AttributeError:
            pass
        return data

    @property
    def primer_apellido(self):
        data = ''
        try:
            data = self.nombre.PrimerApellido.text
        except AttributeError:
            pass
        return data

    @property
    def segundo_apellido(self):
        data = ''
        try:
            data = self.nombre.SegundoApellido.text
        except AttributeError:
            pass
        return data

    @property
    def razon_social(self):
        data = ''
        try:
            data = self.nombre.RazonSocial.text
        except AttributeError:
            pass
        return data


# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from libcomxml.core import XmlModel, XmlField
from gestionatr.output.messages.base_gas import Heading


# Paso 01
class MensajeA103(XmlModel):
    _sort_order = ('mensaje', 'heading', 'a103')

    def __init__(self):
        self.doc_root = None
        self.mensaje = XmlField(
            'sctdapplication', attributes={'xmlns': 'http://localhost/sctd/A103'}
        )
        self.heading = Heading()
        self.a103 = A103()
        super(MensajeA103, self).__init__('sctdapplication', 'mensaje')


class A103(XmlModel):

    _sort_order = ('a103', 'comreferencenum', 'comreferencenumanul', 'reqdate', 'reqhour',
                   'titulartype', 'nationality', 'documenttype', 'documentnum',
                   'cups', 'annulmentreason', 'extrainfo')

    def __init__(self):
        self.a103 = XmlField('a103')
        self.comreferencenum = XmlField('comreferencenum')
        self.comreferencenumanul = XmlField('comreferencenumanul')
        self.reqdate = XmlField('reqdate')
        self.reqhour = XmlField('reqhour')
        self.titulartype = XmlField('titulartype')
        self.nationality = XmlField('nationality')
        self.documenttype = XmlField('documenttype')
        self.documentnum = XmlField('documentnum')
        self.cups = XmlField('cups')
        self.annulmentreason = XmlField('annulmentreason')
        self.extrainfo = XmlField('extrainfo')
        super(A103, self).__init__('a103', 'a103')




# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from .C1 import C1, DatosSolicitud, Contrato, Cliente
from .Deadlines import DeadLine, Workdays, Naturaldays
from gestionatr.utils import get_rec_attr


class C2(C1):
    """Clase que implementa C2."""

    steps = [
        DeadLine('01', Workdays(5)),
        DeadLine('02', Workdays(1)),
        DeadLine('03', Naturaldays(30)),
        DeadLine('05', Workdays(1)),
        DeadLine('06', Workdays(1)),
        DeadLine('08', Workdays(5))
    ]

    # Datos paso 01
    @property
    def datos_solicitud(self):
        tree = '{0}.DatosSolicitud'.format(self._header)
        sol = get_rec_attr(self.obj, tree, False)
        if sol not in [None, False]:
            return DatosSolicitud(sol)
        return False

    @property
    def cliente(self):
        tree = '{0}.Cliente'.format(self._header)
        cli = get_rec_attr(self.obj, tree, False)
        if cli not in [None, False]:
            return Cliente(cli)
        else:
            return False

    @property
    def contrato(self):
        tree = '{0}.Contrato'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return Contrato(data)
        else:
            return False

    @property
    def medida(self):
        tree = '{0}.Medida'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return Medida(data)
        else:
            return False

    @property
    def doc_tecnica(self):
        tree = '{0}.DocTecnica'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DocTecnica(data)
        else:
            return False

    # Datos Paso 03
    @property
    def fecha_prevista_accion(self):
        tree = '{0}.FechaPrevistaAccion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data:
            return data.text
        else:
            return False

    @property
    def fecha_incidencia(self):
        tree = '{0}.FechaIncidencia'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data:
            return data.text
        else:
            return False

    @property
    def incidencias(self):
        tree = '{0}.Incidencia'.format(self._header)
        obj = get_rec_attr(self.obj, tree, False)
        data = []
        if obj not in [None, False]:
            for i in obj:
                data.append(Incidencia(i))
            return data
        return data

    # Datos paso 13
    @property
    def contestacion_incidencia(self):
        tree = '{0}.ContestacionIncidencia'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data:
            return data.text
        else:
            return False

    @property
    def contacto(self):
        tree = '{0}.Contacto'.format(self._header)
        con = get_rec_attr(self.obj, tree, False)
        if con not in [None, False]:
            return Contacto(con)
        else:
            return False

class DatosSolicitud(DatosSolicitud):

    @property
    def tipo_modificacion(self):
        data = ''
        try:
            data = self.datos_solicitud.TipoModificacion.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_solicitud_administrativa(self):
        data = ''
        try:
            data = self.datos_solicitud.TipoSolicitudAdministrativa.text
        except AttributeError:
            pass
        return data

    @property
    def cnae(self):
        data = ''
        try:
            data = self.datos_solicitud.CNAE.text
        except AttributeError:
            pass
        return data

    @property
    def solicitud_tension(self):
        data = ''
        try:
            data = self.datos_solicitud.SolicitudTension.text
        except AttributeError:
            pass
        return data

    @property
    def tension_solicitada(self):
        data = ''
        try:
            data = self.datos_solicitud.TensionSolicitada.text
        except AttributeError:
            pass
        return data


class Contrato(Contrato):

    @property
    def fecha_finalizacion(self):
        data = False
        try:
            data = self.contrato.FechaFinalizacion.text
        except AttributeError:
            pass
        return data

    @property
    def contacto(self):
        data = ''
        try:
            data = Contacto(self.contrato.Contacto)
        except AttributeError:
            pass
        return data

    @property
    def periodicidad_facturacion(self):
        data = ''
        try:
            data = self.contrato.PeriodicidadFacturacion.text
        except AttributeError:
            try:
                data = self.contrato.CondicionesContractuales.PeriodicidadFacturacion.text
            except:
                pass
        return data

    @property
    def consumo_anual_estimado(self):
        data = ''
        try:
            data = self.contrato.ConsumoAnualEstimado.text
        except AttributeError:
            pass
        return data

    @property
    def cups_principal(self):
        data = ''
        try:
            data = self.contrato.CUPSPrincipal.text
        except AttributeError:
            pass
        return data


class Contacto(object):

    def __init__(self, data):
        self.contacto = data

    @property
    def persona_de_contacto(self):
        data = ''
        try:
            data = self.contacto.PersonaDeContacto.text
        except AttributeError:
            pass
        return data

    @property
    def telefonos(self):
        data = []
        try:
            for tel in self.contacto.Telefono:
                data.append((tel.PrefijoPais.text, tel.Numero.text))
        except AttributeError:
            pass
        return data

    @property
    def correo_electronico(self):
        data = ''
        try:
            data = self.contacto.CorreoElectronico.text
        except AttributeError:
            pass
        return data


class Cliente(Cliente):

    @property
    def indicador_tipo_direccion(self):
        data = ''
        try:
            data = self.cliente.IndicadorTipoDireccion.text
        except AttributeError:
            pass
        return data

    @property
    def direccion(self):
        data = ''
        try:
            data = Direccion(self.cliente.Direccion)
        except AttributeError:
            pass
        return data


class Direccion(object):

    def __init__(self, data):
        self.direccion = data

    @property
    def pais(self):
        data = ''
        try:
            data = self.direccion.Pais.text.strip()
        except AttributeError:
            pass
        return data

    @property
    def provincia(self):
        data = ''
        try:
            data = self.direccion.Provincia.text.strip()
        except AttributeError:
            pass
        return data

    @property
    def municipio(self):
        data = ''
        try:
            data = self.direccion.Municipio.text.strip()
        except AttributeError:
            pass
        return data

    @property
    def poblacion(self):
        data = ''
        try:
            data = self.direccion.Poblacion.text.strip()
        except AttributeError:
            pass
        return data

    @property
    def cod_postal(self):
        data = ''
        try:
            data = self.direccion.CodPostal.text.strip()
        except AttributeError:
            pass
        return data

    @property
    def tipo_via(self):
        data = ''
        try:
            data = self.direccion.Via.TipoVia.text.strip()
        except AttributeError:
            try:
                data = self.direccion.TipoVia.text.strip()
            except AttributeError:
                pass
        return data

    @property
    def calle(self):
        data = ''
        try:
            data = self.direccion.Via.Calle.text.strip()
        except AttributeError:
            try:
                data = self.direccion.Calle.text.strip()
            except AttributeError:
                pass
        return data

    @property
    def numero_finca(self):
        data = ''
        try:
            data = self.direccion.Via.NumeroFinca.text.strip()
        except AttributeError:
            try:
                data = self.direccion.NumeroFinca.text.strip()
            except AttributeError:
                pass
        return data

    @property
    def duplicador_finca(self):
        data = ''
        try:
            data = self.direccion.Via.DuplicadorFinca.text.strip()
        except AttributeError:
            try:
                data = self.direccion.DuplicadorFinca.text.strip()
            except AttributeError:
                pass
        return data

    @property
    def escalera(self):
        data = ''
        try:
            data = self.direccion.Via.Escalera.text.strip()
        except AttributeError:
            try:
                data = self.direccion.Escalera.text.strip()
            except AttributeError:
                pass
        return data

    @property
    def piso(self):
        data = ''
        try:
            data = self.direccion.Via.Piso.text.strip()
        except AttributeError:
            try:
                data = self.direccion.Piso.text.strip()
            except AttributeError:
                pass
        return data

    @property
    def puerta(self):
        data = ''
        try:
            data = self.direccion.Via.Puerta.text.strip()
        except AttributeError:
            try:
                data = self.direccion.Puerta.text.strip()
            except AttributeError:
                pass
        return data

    @property
    def tipo_aclarador_finca(self):
        data = ''
        try:
            data = self.direccion.Via.TipoAclaradorFinca.text.strip()
        except AttributeError:
            try:
                data = self.direccion.TipoAclaradorFinca.text.strip()
            except AttributeError:
                pass
        return data

    @property
    def aclarador_finca(self):
        data = ''
        try:
            data = self.direccion.Via.AclaradorFinca.text.strip()
        except AttributeError:
            try:
                data = self.direccion.AclaradorFinca.text.strip()
            except AttributeError:
                pass
        return data

    @property
    def apartado_de_correos(self):
        data = ''
        try:
            data = self.direccion.ApartadoDeCorreos.text.strip()
        except AttributeError:
            pass
        return data


class Medida(object):

    def __init__(self, data):
        self.medida = data

    @property
    def propiedad_equipo(self):
        data = ''
        try:
            data = self.medida.PropiedadEquipo.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_equipo_medida(self):
        data = ''
        try:
            data = self.medida.TipoEquipoMedida.text
        except AttributeError:
            pass
        return data

    @property
    def modelos_aparato(self):
        data = []
        obj = self.medida
        if (hasattr(obj, 'ModelosAparato') and
                hasattr(obj.ModelosAparato, 'ModeloAparato')):
            for d in obj.ModelosAparato.ModeloAparato:
                data.append(ModeloAparato(d))
        return data


class ModeloAparato(object):
    def __init__(self, data):
        self.modelo_aparato = data

    @property
    def tipo_aparato(self):
        data = ''
        try:
            data = self.modelo_aparato.TipoAparato.text
        except AttributeError:
            pass
        return data

    @property
    def marca_aparato(self):
        data = ''
        try:
            data = self.modelo_aparato.MarcaAparato.text
        except AttributeError:
            pass
        return data

    @property
    def modelo_marca(self):
        data = ''
        try:
            data = self.modelo_aparato.ModeloMarca.text
        except AttributeError:
            pass
        return data


class DocTecnica(object):

    def __init__(self, data):
        self.doc_tecnica = data

    @property
    def datos_cie(self):
        data = ''
        try:
            data = DatosCie(self.doc_tecnica.DatosCie)
        except AttributeError:
            pass
        return data

    @property
    def datos_apm(self):
        data = ''
        try:
            data = DatosAPM(self.doc_tecnica.DatosAPM)
        except AttributeError:
            pass
        return data


class DatosCie(object):

    def __init__(self, data):
        self.datos_cie = data

    @property
    def cie_papel(self):
        data = ''
        try:
            data = CIEPapel(self.datos_cie.CIEPapel)
        except AttributeError:
            pass
        return data

    @property
    def cie_electronico(self):
        data = ''
        try:
            data = CIEElectronico(self.datos_cie.CIEElectronico)
        except AttributeError:
            pass
        return data

    @property
    def validez_cie(self):
        data = ''
        try:
            data = self.datos_cie.ValidezCIE.text
        except AttributeError:
            pass
        return data

    @property
    def potencia_no_interrumpible(self):
        data = ''
        try:
            data = self.datos_cie.PotenciaNoInterrumpible.text
        except AttributeError:
            pass
        return data


class CIEPapel(object):

    def __init__(self, data):
        self.cie_papel = data

    @property
    def codigo_cie(self):
        data = ''
        try:
            data = self.cie_papel.CodigoCie.text
        except AttributeError:
            pass
        return data

    @property
    def potencia_inst_bt(self):
        data = ''
        try:
            data = self.cie_papel.PotenciaInstBT.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_emision_cie(self):
        data = False
        try:
            data = self.cie_papel.FechaEmisionCie.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_caducidad_cie(self):
        data = False
        try:
            data = self.cie_papel.FechaCaducidadCie.text
        except AttributeError:
            pass
        return data

    @property
    def nif_instalador(self):
        data = ''
        try:
            data = self.cie_papel.NifInstalador.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_instalador(self):
        data = ''
        try:
            data = self.cie_papel.CodigoInstalador.text
        except AttributeError:
            pass
        return data

    @property
    def tension_suministro_cie(self):
        data = ''
        try:
            data = self.cie_papel.TensionSuministroCIE.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_suministro(self):
        data = ''
        try:
            data = self.cie_papel.TipoSuministro.text
        except AttributeError:
            pass
        return data


class CIEElectronico(object):

    def __init__(self, data):
        self.cie_electronico = data

    @property
    def codigo_cie(self):
        data = ''
        try:
            data = self.cie_electronico.CodigoCie.text
        except AttributeError:
            pass
        return data

    @property
    def sello_electronico(self):
        data = ''
        try:
            data = self.cie_electronico.SelloElectronico.text
        except AttributeError:
            pass
        return data


class DatosAPM(object):

    def __init__(self, data):
        self.datos_apm = data

    @property
    def codigo_apm(self):
        data = ''
        try:
            data = self.datos_apm.CodigoApm.text
        except AttributeError:
            pass
        return data

    @property
    def potencia_inst_at(self):
        data = ''
        try:
            data = self.datos_apm.PotenciaInstAT.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_emision_apm(self):
        data = False
        try:
            data = self.datos_apm.FechaEmisionApm.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_caducidad_apm(self):
        data = False
        try:
            data = self.datos_apm.FechaCaducidadApm.text
        except AttributeError:
            pass
        return data

    @property
    def tension_suministro_apm(self):
        data = ''
        try:
            data = self.datos_apm.TensionSuministroAPM.text
        except AttributeError:
            pass
        return data

    @property
    def nif_instalador(self):
        data = ''
        try:
            data = self.datos_apm.NifInstalador.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_instalador(self):
        data = ''
        try:
            data = self.datos_apm.CodigoInstalador.text
        except AttributeError:
            pass
        return data


class Incidencia(object):

    def __init__(self, data):
        self.incidencia = data

    @property
    def secuencial(self):
        data = ''
        try:
            data = self.incidencia.Secuencial.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_motivo(self):
        data = ''
        try:
            data = self.incidencia.CodigoMotivo.text
        except AttributeError:
            pass
        return data

    @property
    def comentarios(self):
        data = ''
        try:
            data = self.incidencia.Comentarios.text
        except AttributeError:
            pass
        return data



# -*- coding: utf-8 -*-
from libcomxml.core import XmlModel, XmlField
from gestionatr.output.messages.base import Cabecera
from gestionatr.output.messages.sw_c1 import Autoconsumo
from gestionatr.output.messages.sw_c2 import Cliente, Direccion


# Paso 01
class MensajeSolicitudInformacionAlRegistroDePS(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'validacion_cliente')

    def __init__(self):
        self.mensaje = XmlField('MensajeSolicitudInformacionAlRegistroDePS',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.validacion_cliente = ValidacionCliente()
        super(MensajeSolicitudInformacionAlRegistroDePS, self) \
            .__init__('MensajeSolicitudInformacionAlRegistroDePS', 'mensaje')


class ValidacionCliente(XmlModel):

    _sort_order = ('validacion_cliente', 'tipo_identificador', 'identificador')

    def __init__(self):
        self.validacion_cliente = XmlField('ValidacionCliente')
        self.tipo_identificador = XmlField('TipoIdentificador')
        self.identificador = XmlField('Identificador')
        super(ValidacionCliente, self).__init__('ValidacionCliente', 'validacion_cliente')


# Paso 02 accept
class MensajeEnvioInformacionPS(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'envio_informacion_ps')

    def __init__(self):
        self.mensaje = XmlField('MensajeEnvioInformacionPS', attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.envio_informacion_ps = EnvioInformacionPS()
        super(MensajeEnvioInformacionPS, self).__init__('MensajeEnvioInformacionPS', 'mensaje')


class EnvioInformacionPS(XmlModel):

    _sort_order = ('envio_informacion_ps', 'resultado_validacion_cliente', 'en_vigor', 'estado_contratable',
                   'existe_solicitud_en_curso', 'tipo_solicitud_en_curso', 'contrato',
                   'fecha_ultimo_movimiento_tipo_autocons', 'ind_bono_social', 'ind_esencial',
                   'fecha_ultimo_movimiento_ind_esencial','vivienda_habitual','cnae', 'condiciones_contractuales',
                   'modo_facturacion_potencia', 'no_interrumpible', 'potencia_no_interrumpible', 'vas_trafo',
                   'periodicidad_facturacion', 'tipo_de_telegestion', 'icp_activado_telegestion', 'peaje_directo',
                   'deposito_garantia', 'potencia_maxima_autorizada', 'tension_del_suministro', 'derechos_reconocidos',
                   'caracteristicas_pm', 'historia', 'cliente', 'equipo_list', 'doc_tecnica',
                   'expediente_anomalia_fraude', 'expediente_acometida', 'puntos_de_medida')

    def __init__(self):
        self.envio_informacion_ps = XmlField('EnvioInformacionPS')
        self.resultado_validacion_cliente = XmlField('ResultadoValidacionCliente')
        self.en_vigor = XmlField('EnVigor')
        self.estado_contratable = EstadoContratable()
        self.existe_solicitud_en_curso = XmlField('ExisteSolicitudEnCurso')
        self.tipo_solicitud_en_curso = XmlField('TipoSolicitudEnCurso')
        self.contrato = Contrato()
        self.fecha_ultimo_movimiento_tipo_autocons = XmlField('FechaUltimoMovimientoTipoAutocons')
        self.ind_bono_social = XmlField('IndBonoSocial')
        self.ind_esencial = XmlField('IndEsencial')
        self.fecha_ultimo_movimiento_ind_esencial = XmlField('FechaUltimoMovimientoIndEsencial')
        self.vivienda_habitual = XmlField('ViviendaHabitual')
        self.cnae = XmlField('CNAE')
        self.condiciones_contractuales = CondicionesContractuales()
        self.modo_facturacion_potencia = XmlField('ModoFacturacionPotencia')
        self.no_interrumpible = XmlField('NoInterrumpible')
        self.potencia_no_interrumpible = XmlField('PotenciaNoInterrumpible')
        self.vas_trafo = XmlField('VAsTrafo')
        self.periodicidad_facturacion = XmlField('PeriodicidadFacturacion')
        self.tipo_de_telegestion = XmlField('TipodeTelegestion')
        self.icp_activado_telegestion = XmlField('ICPActivadoTelegestion')
        self.peaje_directo = XmlField('PeajeDirecto')
        self.deposito_garantia = XmlField('DepositoGarantia')
        self.potencia_maxima_autorizada = XmlField('PotenciaMaxAutorizada')
        self.tension_del_suministro = XmlField('TensionDelSuministro')
        self.derechos_reconocidos = DerechosReconocidos()
        self.caracteristicas_pm = CaracteristicasPM()
        self.cliente = Cliente()
        self.historia = Historia()
        self.equipo_list = []
        self.doc_tecnica = DocTecnica()
        self.expediente_anomalia_fraude = ExpedienteAnomaliaFraude()
        self.expediente_acometida = ExpedienteAcometida()
        self.puntos_de_medida = PuntosDeMedida()

        super(EnvioInformacionPS, self).__init__('EnvioInformacionPS', 'envio_informacion_ps')


class EstadoContratable(XmlModel):

    _sort_order = ('estado_contratable', 'contratable', 'motivo', 'potencia_max_sin_expediente')

    def __init__(self):
        self.estado_contratable = XmlField('EstadoContratable')
        self.contratable = XmlField('Contratable')
        self.motivo = XmlField('Motivo')
        self.potencia_max_sin_expediente = XmlField('PotenciaMaxSinExpediente')
        super(EstadoContratable, self).__init__('EstadoContratable', 'estado_contratable')


class Cliente(Cliente):
    _sort_order = ('cliente', 'id_cliente', 'nombre', 'indicador_tipo_direccion', 'direccion_suministro', 'telefonos',
                   'correo_electronico')

    def __init__(self):
        super(Cliente, self).__init__()
        self.direccion_suministro = Direccion(name='DireccionSuministro')


class Contrato(XmlModel):

    _sort_order = ('contrato', 'tipo_contrato_atr', 'fecha_finalizacion', 'autoconsumo',)

    def __init__(self):
        self.contrato = XmlField('Contrato')
        self.tipo_contrato_atr = XmlField('TipoContratoATR')
        self.fecha_finalizacion = XmlField('FechaFinalizacion')
        self.autoconsumo = Autoconsumo()

        super(Contrato, self).__init__('Contrato', 'contrato')


class CondicionesContractuales(XmlModel):

    _sort_order = ('condiciones_contractuales', 'tarifa_atr', 'potencias_contratadas', 'modo_control_potencia')

    def __init__(self):
        self.condiciones_contractuales = XmlField('CondicionesContractuales')
        self.tarifa_atr = XmlField('TarifaATR')
        self.potencias_contratadas = PotenciasContratadas()
        self.modo_control_potencia = XmlField('ModoControlPotencia')
        super(CondicionesContractuales, self).__init__('CondicionesContractuales', 'condiciones_contractuales')


class PotenciasContratadas(XmlModel):
    _sort_order = ('potencies', 'p1', 'p2', 'p3', 'p4', 'p5', 'p6')

    def __init__(self):
        self.potencies = XmlField('PotenciasContratadas')
        self.p1 = XmlField('Potencia', attributes={'Periodo': '1'}, rep=lambda x: '%d' % x)
        self.p2 = XmlField('Potencia', attributes={'Periodo': '2'})
        self.p3 = XmlField('Potencia', attributes={'Periodo': '3'})
        self.p4 = XmlField('Potencia', attributes={'Periodo': '4'})
        self.p5 = XmlField('Potencia', attributes={'Periodo': '5'})
        self.p6 = XmlField('Potencia', attributes={'Periodo': '6'})
        super(PotenciasContratadas, self).__init__('PotenciasContratadas', 'potencies')


class DerechosReconocidos(XmlModel):

    _sort_order = ('derechos_reconocidos', 'derecho_acceso', 'derechos_extension', 'fecha_limite_derechos_extension')

    def __init__(self):
        self.derechos_reconocidos = XmlField('DerechosReconocidos')
        self.derecho_acceso = XmlField('DerechoAcceso')
        self.derechos_extension = XmlField('DerechosExtension')
        self.fecha_limite_derechos_extension = XmlField('FechaLimiteDerechosExtension')
        super(DerechosReconocidos, self).__init__('DerechosReconocidos', 'derechos_reconocidos')


class CaracteristicasPM(XmlModel):

    _sort_order = ('caracteristicas_pm', 'tipo_pm', 'tension_pm')

    def __init__(self):
        self.caracteristicas_pm = XmlField('CaracteristicasPM')
        self.tipo_pm = XmlField('TipoPM')
        self.tension_pm = XmlField('TensionPM')
        super(CaracteristicasPM, self).__init__('CaracteristicasPM', 'caracteristicas_pm')


class Historia(XmlModel):

    _sort_order = ('historia', 'fecha_ultimo_movimiento_contratacion', 'fecha_cambio_comercializador',
                   'fecha_ultima_lectura', 'fecha_ultima_verificacion', 'resultado_ultima_lectura')

    def __init__(self):
        self.historia = XmlField('Historia')
        self.fecha_ultimo_movimiento_contratacion = XmlField('FechaUltimoMovimientoContratacion')
        self.fecha_cambio_comercializador = XmlField('FechaCambioComercializador')
        self.fecha_ultima_lectura = XmlField('FechaUltimaLectura')
        self.fecha_ultima_verificacion = XmlField('FechaUltimaVerificacion')
        self.resultado_ultima_lectura = XmlField('ResultadoUltimaVerificacion')
        super(Historia, self).__init__('Historia', 'historia')


class Equipo(XmlModel):

    _sort_order = ('equipo', 'tipo_aparato', 'tipo_equipo', 'tipo_propiedad',
                   'codigo_fases_equipo_medida', 'tipo_dh_edm', 'relacion_transformacion_intensidad')

    def __init__(self):
        self.equipo = XmlField('Equipo')
        self.tipo_aparato = XmlField('TipoAparato')
        self.tipo_equipo = XmlField('TipoEquipo')
        self.tipo_propiedad = XmlField('TipoPropiedad')
        self.codigo_fases_equipo_medida = XmlField('CodigoFasesEquipoMedida')
        self.tipo_dh_edm = XmlField('TipoDHEdM')
        self.relacion_transformacion_intensidad = XmlField('RelacionTransformacionIntensidad')
        super(Equipo, self).__init__('Equipo', 'equipo')


class DocTecnica(XmlModel):

    _sort_order = ('doc_tecnica', 'cie_disponible', 'datos_cie', 'datos_apm')

    def __init__(self):
        self.doc_tecnica = XmlField('DocTecnica')
        self.cie_disponible = XmlField('CIEDisponible')
        self.datos_cie = DatosCie()
        self.datos_apm = DatosAPM()
        super(DocTecnica, self).__init__('DocTecnica', 'doc_tecnica')


class DatosCie(XmlModel):

    _sort_order = ('datos_cie', 'potencia_inst_bt', 'cie_papel', 'cie_electronico', 'validez_cie')

    def __init__(self):
        self.datos_cie = XmlField('DatosCie')
        self.potencia_inst_bt = XmlField('PotenciaInstBT')
        self.cie_papel = CIEPapel()
        self.cie_electronico = CIEElectronico()
        self.validez_cie = XmlField('ValidezCIE')
        super(DatosCie, self).__init__('DatosCie', 'datos_cie')


class CIEPapel(XmlModel):

    _sort_order = ('cie_papel', 'codigo_cie', 'fecha_emision_cie', 'fecha_caducidad_cie',
                   'tension_suministro_cie', 'tipo_suministro')

    def __init__(self):
        self.cie_papel = XmlField('CIEPapel')
        self.codigo_cie = XmlField('CodigoCie')
        self.fecha_emision_cie = XmlField('FechaEmisionCie')
        self.fecha_caducidad_cie = XmlField('FechaCaducidadCie')
        self.tension_suministro_cie = XmlField('TensionSuministroCIE')
        self.tipo_suministro = XmlField('TipoSuministro')
        super(CIEPapel, self).__init__('CIEPapel', 'cie_papel')


class CIEElectronico(XmlModel):
    _sort_order = ('cie_electronico', 'cie_papel', 'cie_electronico', 'validez_cie')

    def __init__(self):
        self.cie_electronico = XmlField('CIEElectronico')
        self.codigo_cie = XmlField('CodigoCie')
        self.sello_electronico = XmlField('SelloElectronico')
        super(CIEElectronico, self).__init__('CIEElectronico', 'cie_electronico')


class DatosAPM(XmlModel):

    _sort_order = ('datos_apm', 'codigo_apm', 'potencia_inst_at', 'fecha_emision_apm', 'fecha_caducidad_apm')

    def __init__(self):
        self.datos_apm = XmlField('DatosAPM')
        self.codigo_apm = XmlField('CodigoApm')
        self.potencia_inst_at = XmlField('PotenciaInstAT')
        self.fecha_emision_apm = XmlField('FechaEmisionApm')
        self.fecha_caducidad_apm = XmlField('FechaCaducidadApm')
        super(DatosAPM, self).__init__('DatosAPM', 'datos_apm')


class ExpedienteAnomaliaFraude(XmlModel):

    _sort_order = ('expediente_anomalia_fraude', 'expediente_abierto', 'codigo_motivo_expediente')

    def __init__(self):
        self.expediente_anomalia_fraude = XmlField('ExpedienteAnomaliaFraude')
        self.expediente_abierto = XmlField('ExpedienteAbierto')
        self.codigo_motivo_expediente = XmlField('CdigoMotivoExpediente')
        super(ExpedienteAnomaliaFraude, self).__init__('ExpedienteAnomaliaFraude', 'expediente_anomalia_fraude')


class ExpedienteAcometida(XmlModel):

    _sort_order = ('expediente_acometida', 'expediente_abierto', 'codigo_motivo_expediente')

    def __init__(self):
        self.expediente_acometida = XmlField('ExpedienteAcometida')
        self.expediente_abierto = XmlField('ExpedienteAbierto')
        self.codigo_motivo_expediente = XmlField('CdigoMotivoExpediente')
        super(ExpedienteAcometida, self).__init__('ExpedienteAcometida', 'expediente_acometida')


class PuntosDeMedida(XmlModel):

    _sort_order = ('puntos_de_medida', 'puntos_de_medida_list')

    def __init__(self):
        self.puntos_de_medida = XmlField('PuntosDeMedida')
        self.puntos_de_medida_list = []
        super(PuntosDeMedida, self).__init__('PuntosDeMedida', 'puntos_de_medida')


class PuntoDeMedida(XmlModel):

    _sort_order = ('punto_de_medida', 'direccion_enlace', 'telefono_telemedida', 'clave_acceso')

    def __init__(self):
        self.punto_de_medida = XmlField('PuntoDeMedida')
        self.direccion_enlace = XmlField('DireccionEnlace')
        self.telefono_telemedida = XmlField('TelefonoTelemedida')
        self.clave_acceso = XmlField('ClaveAcceso')
        super(PuntoDeMedida, self).__init__('PuntoDeMedida', 'punto_de_medida')


# Paso 02 (Rechazo)
class MensajeRechazo(XmlModel):

    _sort_order = ('mensaje_rechazo', 'cabecera', 'rechazos_peticion')

    def __init__(self):
        self.mensaje_rechazo = XmlField('MensajeRechazoP0',
                                        attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.rechazos_peticion = RechazosPeticion()
        super(MensajeRechazo, self).__init__('MensajeRechazoP0', 'mensaje_rechazo')


class RechazosPeticion(XmlModel):

    _sort_order = ('rechazos_peticion', 'fecha_rechazo', 'rechazo')

    def __init__(self):
        self.rechazos_peticion = XmlField('RechazosPeticion')
        self.fecha_rechazo = XmlField('FechaRechazo')
        self.rechazo = Rechazo()
        super(RechazosPeticion, self).__init__('RechazosPeticion', 'rechazos_peticion')


class Rechazo(XmlModel):

    _sort_order = ('rechazo', 'secuencial', 'codigo_motivo', 'comentarios')

    def __init__(self):
        self.rechazo = XmlField('Rechazo')
        self.secuencial = XmlField('Secuencial')
        self.codigo_motivo = XmlField('CodigoMotivo')
        self.comentarios = XmlField('Comentarios')
        super(Rechazo, self).__init__('Rechazo', 'rechazo')


ENVELOPE_TEMPLATE_AMB_NAMESPACE_AL_PAS = """
<soap:Envelope 
xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:eleg="http://localhost/elegibilidad"
>
    <soap:Header />
    <soap:Body>
        <eleg:sync>
            <eleg:Paso01>
                {xml_str}
            </eleg:Paso01>
        </eleg:sync>
    </soap:Body>
</soap:Envelope>
"""

ENVELOPE_TEMPLATE_SENSE_NAMESPACE_AL_PAS01 = """
<soap:Envelope 
xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:eleg="http://localhost/elegibilidad"
>
    <soap:Header />
    <soap:Body>
        <eleg:sync>
            <Paso01>
                {xml_str}
            </Paso01>
        </eleg:sync>
    </soap:Body>
</soap:Envelope>
"""


ENVELOP_BY_DISTR = {
    '0021': {
        'template': ENVELOPE_TEMPLATE_SENSE_NAMESPACE_AL_PAS01,
        'extra_headers': {},
    },
    '0031': {
        'template': ENVELOPE_TEMPLATE_SENSE_NAMESPACE_AL_PAS01,
        'extra_headers': {},
    },
    '0022': {
        'template': ENVELOPE_TEMPLATE_SENSE_NAMESPACE_AL_PAS01,
        'extra_headers': {},
    },
    'altres': {
        'template': ENVELOPE_TEMPLATE_SENSE_NAMESPACE_AL_PAS01,
        'extra_headers': {
            "SOAPAction": ""
        },
    },
    'reintent_2': {  # Els de CIDE volen el soap action. Com que son molts en comptes de anarlos definint un a un fem un reintent amb aquest header sempre
        'template': ENVELOPE_TEMPLATE_SENSE_NAMESPACE_AL_PAS01,
        'extra_headers': {
            "SOAPAction": "http://localhost/elegibilidad/Sync/syncRequest"
        },
    },
    'reintent': {
        'template': ENVELOPE_TEMPLATE_AMB_NAMESPACE_AL_PAS,
        'extra_headers': {
            "SOAPAction": ""
        },
    },
}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from gestionatr.output.messages.sw_m1 import *

# Paso 02 -> C1 MensajeRechazo


# Paso 05
class MensajeActivacionModificacionDeATRUnidir(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'activacion_modificaciones')

    def __init__(self):
        self.mensaje = XmlField('MensajeActivacionModificacionDeATRUnidir',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.activacion_modificaciones = ActivacionModificaciones()
        super(MensajeActivacionModificacionDeATRUnidir, self).__init__(
            'MensajeActivacionModificacionDeATRUnidir', 'mensaje'
        )


class ActivacionModificaciones(XmlModel):

    _sort_order = ('activacion_modificaciones', 'datos_activacion', 'ind_esencial',
                   'fecha_ultimo_movimiento_ind_esencial', 'contrato', 'puntos_de_medida')

    def __init__(self):
        self.activacion_modificaciones = XmlField('ActivacionModificaciones')
        self.datos_activacion = DatosActivacion()
        self.ind_esencial = XmlField('IndEsencial')
        self.fecha_ultimo_movimiento_ind_esencial = XmlField('FechaUltimoMovimientoIndEsencial')
        self.contrato = Contrato()
        self.puntos_de_medida = PuntosDeMedida()
        super(ActivacionModificaciones, self).__init__('ActivacionModificaciones', 'activacion_modificaciones')


class DatosActivacion(DatosActivacion):

    _sort_order = ('datos_activacion', 'motivo_activacion_list', 'fecha', 'bono_social', 'ind_esencial',
                   'fecha_ultimo_movimiento_ind_esencial')

    def __init__(self):
        super(DatosActivacion, self).__init__()
        self.motivo_activacion_list = []


class MotivoActivacionUnidireccional(XmlModel):

    _sort_order = ('motivo_activacion_unidireccional', 'motivo')

    def __init__(self):
        self.motivo_activacion_unidireccional = XmlField('MotivoActivacionUnidireccional')
        self.motivo = XmlField('Motivo')
        super(MotivoActivacionUnidireccional, self).__init__(
            'MotivoActivacionUnidireccional', 'motivo_activacion_unidireccional'
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from .C2 import C2, Cliente, Contacto
from .C1 import DatosAceptacion
from .W1 import LecturaAportada
from .Deadlines import DeadLine, Workdays, Naturaldays
from gestionatr.utils import get_rec_attr
from gestionatr.defs import SUBTYPES_R101


class R1(C2):
    """Clase que implementa R1."""

    steps = [
        DeadLine('01', Workdays(5)),
        DeadLine('02', Workdays(15)),
        DeadLine('03', Naturaldays(20)),
    ]

    # Datos paso 01
    @property
    def datos_solicitud(self):
        tree = '{0}.DatosSolicitud'.format(self._header)
        sol = get_rec_attr(self.obj, tree, False)
        if sol not in [None, False]:
            return DatosSolicitud(sol)
        else:
            return False

    @property
    def variables_detalle_reclamacion(self):
        tree = '{0}.VariablesDetalleReclamacion'.format(self._header)
        obj = get_rec_attr(self.obj, tree, False)
        data = []
        if obj not in [None, False] and hasattr(obj, "VariableDetalleReclamacion"):
            for i in obj.VariableDetalleReclamacion:
                data.append(VariableDetalleReclamacion(i))
            return data
        return data

    @property
    def tipo_reclamante(self):
        tree = '{0}.TipoReclamante'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data:
            return data.text
        else:
            return False

    @property
    def reclamante(self):
        tree = '{0}.Reclamante'.format(self._header)
        cli = get_rec_attr(self.obj, tree, False)
        if cli not in [None, False]:
            return Reclamante(cli)
        else:
            return False

    # Datos paso 02 aceptacion
    @property
    def datos_aceptacion(self):
        tree = '{0}.DatosAceptacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DatosAceptacion(data)
        else:
            return False

    # Datos paso 03
    @property
    def datos_informacion(self):
        tree = '{0}.DatosInformacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DatosInformacion(data)
        else:
            return False

    @property
    def informacion_intermedia(self):
        tree = '{0}.InformacionIntermedia'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return InformacionIntermedia(data)
        else:
            return False

    @property
    def retipificacion(self):
        tree = '{0}.Retipificacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return Retipificacion(data)
        else:
            return False

    @property
    def solicitudes_informacion_adicional(self):
        data = []
        tree = '{0}.SolicitudesInformacionAdicional'.format(self._header)
        obj = get_rec_attr(self.obj, tree, False)
        if obj not in [None, False]:
            sol = get_rec_attr(obj, 'SolicitudInformacionAdicional', False)
            if sol not in [None, False]:
                for d in sol:
                    data.append(SolicitudInformacionAdicional(d))
        return data

    @property
    def solicitud_informacion_adicional_para_retipificacion(self):
        tree = '{0}.SolicitudesInformacionAdicional.SolicitudInformacionAdicionalParaRetipificacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return SolicitudInformacionAdicionalParaRetipificacion(data)
        else:
            return False

    @property
    def parametros_comunicacion(self):
        data = []
        tree = '{0}.ParametrosComunicacion'.format(self._header)
        obj = get_rec_attr(self.obj, tree, False)
        if obj not in [None, False]:
            for d in obj:
                data.append(ParametrosComunicacion(d))
        return data

    # Datos paso 04
    @property
    def datos_envio_informacion(self):
        tree = '{0}.DatosEnvioInformacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DatosEnvioInformacion(data)
        else:
            return False

    @property
    def variables_aportacion_informacion(self):
        data = []
        tree = '{0}.VariablesAportacionInformacion'.format(self._header)
        obj = get_rec_attr(self.obj, tree, False)
        if obj not in [None, False]:
            for d in obj.VariableAportacionInformacion:
                data.append(VariableAportacionInformacion(d))
        return data

    @property
    def variables_aportacion_informacion_para_retipificacion(self):
        data = []
        tree = '{0}.VariablesAportacionInformacionParaRetipificacion'.format(self._header)
        obj = get_rec_attr(self.obj, tree, False)
        if obj not in [None, False]:
            for d in obj.VariableAportacionInformacionParaRetipificacion:
                data.append(VariableDetalleReclamacion(d))
        return data

    # Datos paso 5
    @property
    def datos_cierre(self):
        tree = '{0}.DatosCierre'.format(self._header)
        sol = get_rec_attr(self.obj, tree, False)
        if sol not in [None, False]:
            return DatosCierre(sol)
        else:
            return False

    @property
    def cod_contrato(self):
        tree = '{0}.CodContrato'.format(self._header)
        sol = get_rec_attr(self.obj, tree, False)
        if sol:
            return sol.text
        else:
            return False

    # Campos Minimos
    def get_subtypes(self):
        r1_type = self.datos_solicitud.tipo
        return [x['code'] for x in SUBTYPES_R101 if x['type'] == r1_type]

    def get_type_from_subtype(self):
        r1_subtype = self.datos_solicitud.subtipo
        for x in SUBTYPES_R101:
            if x['code'] == r1_subtype:
                return x['type']
        return []

    def get_minimum_fields(self):
        subtype = self.datos_solicitud.subtipo
        for x in SUBTYPES_R101:
            if x['code'] == subtype:
                return x['min_fields']
        return []

    def check_minimum_fields(self):
        checker = MinimumFieldsChecker(self)
        return checker.check()


class DatosSolicitud(object):

    def __init__(self, data):
        self.datos_solicitud = data

    @property
    def tipo(self):
        data = ''
        try:
            data = self.datos_solicitud.Tipo.text
        except AttributeError:
            pass
        return data

    @property
    def subtipo(self):
        data = ''
        try:
            data = self.datos_solicitud.Subtipo.text
        except AttributeError:
            pass
        return data

    @property
    def referencia_origen(self):
        data = ''
        try:
            data = self.datos_solicitud.ReferenciaOrigen.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_limite(self):
        data = False
        try:
            data = self.datos_solicitud.FechaLimite.text
        except AttributeError:
            pass
        return data

    @property
    def prioritario(self):
        data = ''
        try:
            data = self.datos_solicitud.Prioritario.text
        except AttributeError:
            pass
        return data


class Reclamante(Cliente):

    @property
    def tipo_identificador(self):
        data = ''
        try:
            data = self.cliente.IdReclamante.TipoIdentificador.text
        except AttributeError:
            pass
        return data

    @property
    def identificador(self):
        data = ''
        try:
            data = self.cliente.IdReclamante.Identificador.text
        except AttributeError:
            pass
        return data


class VariableDetalleReclamacion(object):

    def __init__(self, data):
        self.variable = data

    @property
    def num_expediente_acometida(self):
        data = ''
        try:
            data = self.variable.NumExpedienteAcometida.text
        except AttributeError:
            pass
        return data

    @property
    def num_expediente_fraude(self):
        data = ''
        try:
            data = self.variable.NumExpedienteFraude.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_incidente(self):
        data = False
        try:
            data = self.variable.FechaIncidente.text
        except AttributeError:
            pass
        return data

    @property
    def num_factura_atr(self):
        data = ''
        try:
            data = self.variable.NumFacturaATR.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_concepto_facturado(self):
        data = ''
        try:
            data = self.variable.TipoConceptoFacturado.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_lectura(self):
        data = False
        try:
            data = self.variable.FechaLectura.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_dhedm(self):
        data = ''
        try:
            data = self.variable.TipoDHEdM.text
        except AttributeError:
            pass
        return data

    @property
    def cau(self):
        data = ''
        try:
            data = self.variable.CAU.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_incidencia(self):
        data = ''
        try:
            data = self.variable.CodigoIncidencia.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_solicitud(self):
        data = ''
        try:
            data = self.variable.CodigoSolicitud.text
        except AttributeError:
            pass
        return data

    @property
    def parametro_contratacion(self):
        data = ''
        try:
            data = self.variable.ParametroContratacion.text
        except AttributeError:
            pass
        return data

    @property
    def concepto_disconformidad(self):
        data = ''
        try:
            data = self.variable.ConceptoDisconformidad.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_de_atencion_incorrecta(self):
        data = ''
        try:
            data = self.variable.TipoDeAtencionIncorrecta.text
        except AttributeError:
            pass
        return data

    @property
    def motivo_consulta(self):
        data = ''
        try:
            data = self.variable.MotivoConsulta.text
        except AttributeError:
            pass
        return data

    @property
    def iban(self):
        data = ''
        try:
            data = self.variable.IBAN.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_solicitud_reclamacion(self):
        data = ''
        try:
            data = self.variable.CodigoSolicitudReclamacion.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_desde(self):
        data = False
        try:
            data = self.variable.FechaDesde.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_hasta(self):
        data = False
        try:
            data = self.variable.FechaHasta.text
        except AttributeError:
            pass
        return data

    @property
    def importe_reclamado(self):
        data = ''
        try:
            data = self.variable.ImporteReclamado.text
        except AttributeError:
            pass
        return data

    @property
    def lecturas_aportadas(self):
        data = []
        obj = self.variable
        if (hasattr(obj, 'LecturasAportadas') and
                hasattr(obj.LecturasAportadas, 'LecturaAportada')):
            for d in obj.LecturasAportadas.LecturaAportada:
                data.append(LecturaAportada(d))
        return data

    @property
    def disconformidad_autoconsumo(self):
        data = []
        obj = self.variable
        if (hasattr(obj, 'DisconformidadAutoconsumo') and
                hasattr(obj.DisconformidadAutoconsumo, 'TipoDisconformidadAutoconsumo')):
            for d in obj.DisconformidadAutoconsumo.TipoDisconformidadAutoconsumo:
                data.append(d.text)
        return data

    @property
    def contacto(self):
        data = ''
        try:
            data = Contacto(self.variable.Contacto)
        except AttributeError:
            pass
        return data

    @property
    def ubicacion_incidencia(self):
        data = ''
        try:
            data = UbicacionIncidencia(self.variable.UbicacionIncidencia)
        except AttributeError:
            pass
        return data


class UbicacionIncidencia(object):

    def __init__(self, data):
        self.ubicacion = data

    @property
    def des_ubicacion_incidencia(self):
        data = ''
        try:
            data = self.ubicacion.DesUbicacionIncidencia.text
        except AttributeError:
            pass
        return data

    @property
    def provincia(self):
        data = ''
        try:
            data = self.ubicacion.Provincia.text
        except AttributeError:
            pass
        return data

    @property
    def municipio(self):
        data = ''
        try:
            data = self.ubicacion.Municipio.text
        except AttributeError:
            pass
        return data

    @property
    def poblacion(self):
        data = ''
        try:
            data = self.ubicacion.Poblacion.text
        except AttributeError:
            pass
        return data

    @property
    def cod_postal(self):
        data = ''
        try:
            data = self.ubicacion.CodPostal.text
        except AttributeError:
            pass
        return data


class LecturaAportada(LecturaAportada):

    @property
    def codigo_periodo_dh(self):
        data = ''
        try:
            data = self.lectura.CodigoPeriodoDH.text
        except AttributeError:
            pass
        return data


class DatosAceptacion(DatosAceptacion):

    @property
    def codigo_reclamacion_distribuidora(self):
        data = ''
        try:
            data = self.datos_aceptacion.CodigoReclamacionDistribuidora.text
        except AttributeError:
            pass
        return data


class DatosInformacion(object):

    def __init__(self, data):
        self.datos_informacion = data

    @property
    def num_expediente_acometida(self):
        data = ''
        try:
            data = self.datos_informacion.NumExpedienteAcometida.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_comunicacion(self):
        data = ''
        try:
            data = self.datos_informacion.TipoComunicacion.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_reclamacion_distribuidora(self):
        data = ''
        try:
            data = self.datos_informacion.CodigoReclamacionDistribuidora.text
        except AttributeError:
            pass
        return data


class InformacionIntermedia(object):

    def __init__(self, data):
        self.info_intermedia = data

    @property
    def desc_informacion_intermedia(self):
        data = ''
        try:
            data = self.info_intermedia.DescInformacionIntermedia.text
        except AttributeError:
            pass
        return data

    @property
    def intervenciones(self):
        data = ''
        try:
            data = self.info_intermedia.Intervenciones.text
        except AttributeError:
            pass
        return data

    @property
    def intervenciones(self):
        data = []
        obj = self.info_intermedia
        if (hasattr(obj, 'Intervenciones') and
                hasattr(obj.Intervenciones, 'Intervencion')):
            for d in obj.Intervenciones.Intervencion:
                data.append(Intervencion(d))
        return data


class Intervencion(object):

    def __init__(self, data):
        self.intervencion = data

    @property
    def tipo_intervencion(self):
        data = ''
        try:
            data = self.intervencion.TipoIntervencion.text
        except AttributeError:
            pass
        return data

    @property
    def fecha(self):
        data = False
        try:
            data = self.intervencion.Fecha.text
        except AttributeError:
            pass
        return data

    @property
    def hora_desde(self):
        data = ''
        try:
            data = self.intervencion.HoraDesde.text
        except AttributeError:
            pass
        return data

    @property
    def hora_hasta(self):
        data = ''
        try:
            data = self.intervencion.HoraHasta.text
        except AttributeError:
            pass
        return data

    @property
    def numero_visita(self):
        data = ''
        try:
            data = self.intervencion.NumeroVisita.text
        except AttributeError:
            pass
        return data

    @property
    def resultado(self):
        data = ''
        try:
            data = self.intervencion.Resultado.text
        except AttributeError:
            pass
        return data

    @property
    def detalle_resultado(self):
        data = ''
        try:
            data = self.intervencion.DetalleResultado.text
        except AttributeError:
            pass
        return data


class Retipificacion(object):

    def __init__(self, data):
        self.retipificacion = data

    @property
    def tipo(self):
        data = ''
        try:
            data = self.retipificacion.Tipo.text
        except AttributeError:
            pass
        return data

    @property
    def subtipo(self):
        data = ''
        try:
            data = self.retipificacion.Subtipo.text
        except AttributeError:
            pass
        return data

    @property
    def desc_retipificacion(self):
        data = ''
        try:
            data = self.retipificacion.DescRetipificacion.text
        except AttributeError:
            pass
        return data


class SolicitudInformacionAdicional(object):

    def __init__(self, data):
        self.sol = data

    @property
    def tipo_informacion_adicional(self):
        data = ''
        try:
            data = self.sol.TipoInformacionAdicional.text
        except AttributeError:
            pass
        return data

    @property
    def desc_peticion_informacion(self):
        data = ''
        try:
            data = self.sol.DescPeticionInformacion.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_limite_envio(self):
        data = False
        try:
            data = self.sol.FechaLimiteEnvio.text
        except AttributeError:
            pass
        return data


class SolicitudInformacionAdicionalParaRetipificacion(object):

    def __init__(self, data):
        self.sol = data

    @property
    def tipo(self):
        data = ''
        try:
            data = self.sol.Tipo.text
        except AttributeError:
            pass
        return data

    @property
    def subtipo(self):
        data = ''
        try:
            data = self.sol.Subtipo.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_limite_envio(self):
        data = False
        try:
            data = self.sol.FechaLimiteEnvio.text
        except AttributeError:
            pass
        return data


class ParametrosComunicacion(object):

    def __init__(self, data):
        self.sol = data

    @property
    def cod_pm(self):
        data = ''
        try:
            data = self.sol.CodPM.text
        except AttributeError:
            pass
        return data

    @property
    def cod_pm_principal(self):
        data = ''
        try:
            data = self.sol.CodPMPrincipal.text
        except AttributeError:
            pass
        return data

    @property
    def funcion(self):
        data = False
        try:
            data = self.sol.Funcion.text
        except AttributeError:
            pass
        return data

    @property
    def direccion_enlace(self):
        data = False
        try:
            data = self.sol.DireccionEnlace.text
        except AttributeError:
            pass
        return data

    @property
    def num_linea(self):
        data = False
        try:
            data = self.sol.NumLinea.text
        except AttributeError:
            pass
        return data

    @property
    def clave_acceso(self):
        data = False
        try:
            data = self.sol.ClaveAcceso.text
        except AttributeError:
            pass
        return data
    @property
    def clave_lectura(self):
        data = False
        try:
            data = self.sol.ClaveLectura.text
        except AttributeError:
            pass
        return data

    @property
    def telefono_telemedida(self):
        data = False
        try:
            data = self.sol.Telefono.TelefonoTelemedida.text
        except AttributeError:
            pass
        return data

    @property
    def telefono_velocidad_comunicacion(self):
        data = False
        try:
            data = self.sol.Telefono.VelocidadComunicacion.text
        except AttributeError:
            pass
        return data

    @property
    def telefono_bit_datos(self):
        data = False
        try:
            data = self.sol.Telefono.BitDatos.text
        except AttributeError:
            pass
        return data

    @property
    def telefono_paridad(self):
        data = False
        try:
            data = self.sol.Telefono.Paridad.text
        except AttributeError:
            pass
        return data

    @property
    def telefono_bit_stop(self):
        data = False
        try:
            data = self.sol.Telefono.BitStop.text
        except AttributeError:
            pass
        return data

    @property
    def ip_direccion(self):
        data = False
        try:
            data = self.sol.IP.DireccionIP.text
        except AttributeError:
            pass
        return data

    @property
    def ip_puerto_enlace(self):
        data = False
        try:
            data = self.sol.IP.PuertoEnlace.text
        except AttributeError:
            pass
        return data


class DatosEnvioInformacion(object):

    def __init__(self, data):
        self.datos = data

    @property
    def num_expediente_acometida(self):
        data = ''
        try:
            data = self.datos.NumExpedienteAcometida.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_informacion(self):
        data = False
        try:
            data = self.datos.FechaInformacion.text
        except AttributeError:
            pass
        return data


class VariableAportacionInformacion(object):

    def __init__(self, data):
        self.var = data

    @property
    def tipo_informacion(self):
        data = ''
        try:
            data = self.var.TipoInformacion.text
        except AttributeError:
            pass
        return data

    @property
    def desc_peticion_informacion(self):
        data = ''
        try:
            data = self.var.DescPeticionInformacion.text
        except AttributeError:
            pass
        return data

    @property
    def variable(self):
        data = ''
        try:
            data = self.var.Variable.text
        except AttributeError:
            pass
        return data

    @property
    def valor(self):
        data = ''
        try:
            data = self.var.Valor.text
        except AttributeError:
            pass
        return data


class DatosCierre(object):

    def __init__(self, data):
        self.datos = data

    @property
    def num_expediente_acometida(self):
        data = ''
        try:
            data = self.datos.NumExpedienteAcometida.text
        except AttributeError:
            pass
        return data

    @property
    def fecha(self):
        data = False
        try:
            data = self.datos.Fecha.text
        except AttributeError:
            pass
        return data

    @property
    def hora(self):
        data = ''
        try:
            data = self.datos.Hora.text
        except AttributeError:
            pass
        return data

    @property
    def tipo(self):
        data = ''
        try:
            data = self.datos.Tipo.text
        except AttributeError:
            pass
        return data

    @property
    def subtipo(self):
        data = ''
        try:
            data = self.datos.Subtipo.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_reclamacion_distribuidora(self):
        data = ''
        try:
            data = self.datos.CodigoReclamacionDistribuidora.text
        except AttributeError:
            pass
        return data

    @property
    def resultado_reclamacion(self):
        data = ''
        try:
            data = self.datos.ResultadoReclamacion.text
        except AttributeError:
            pass
        return data

    @property
    def detalle_resultado(self):
        data = ''
        try:
            data = self.datos.DetalleResultado.text
        except AttributeError:
            pass
        return data

    @property
    def observaciones(self):
        data = ''
        try:
            data = self.datos.Observaciones.text
        except AttributeError:
            pass
        return data

    @property
    def indemnizacion_abonada(self):
        data = ''
        try:
            data = self.datos.IndemnizacionAbonada.text
        except AttributeError:
            pass
        return data

    @property
    def num_expediente_anomalia_fraude(self):
        data = ''
        try:
            data = self.datos.NumExpedienteAnomaliaFraude.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_movimiento(self):
        data = False
        try:
            data = self.datos.FechaMovimiento.text
        except AttributeError:
            pass
        return data


class MinimumFieldsChecker(object):

    def __init__(self, r1):
        self.r1 = r1

    def check(self):
        errors = []
        for field in self.r1.get_minimum_fields():
            valid = getattr(self, 'check_{0}'.format(field), None)
            if not valid():
                errors.append(field)
        return errors

    def check_tipo_disconforme_autoconsumo(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.disconformidad_autoconsumo:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_cau(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.cau:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_motivo_consulta(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.motivo_consulta:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_nif_cliente(self):
        return get_rec_attr(self.r1, "cliente.identificador", False)

    def check_nombre_cliente(self):
        return (get_rec_attr(self.r1, "cliente.nombre_de_pila", False) and
                get_rec_attr(self.r1, "cliente.primer_apellido", False)) \
               or get_rec_attr(self.r1, "cliente.razon_social", False)

    def check_telefono_contacto(self):
        if self.r1.cliente and len(self.r1.cliente.telefonos) > 0:
            return True
        for var in self.r1.variables_detalle_reclamacion:
            if var.contacto and len(var.contacto.telefonos) > 0:
                return True
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_cups(self):
        return get_rec_attr(self.r1, "cups", False)

    def check_fecha_incidente(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.fecha_incidente:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_comentarios(self):
        return self.r1.comentarios

    def check_codigo_incidencia(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.codigo_incidencia:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_persona_de_contacto(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.contacto:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_num_fact(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.num_factura_atr:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_tipo_concepto_facturado(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.tipo_concepto_facturado:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_lectura(self):
        for var in self.r1.variables_detalle_reclamacion:
            if len(var.lecturas_aportadas) == 0:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_fecha_de_lectura(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.fecha_lectura:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_fecha_desde(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.fecha_desde:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_fecha_hasta(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.fecha_hasta:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_ubicacion_incidencia(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.ubicacion_incidencia:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_codigo_de_solicitud(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.codigo_solicitud:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_concepto_contratacion(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.parametro_contratacion:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_cta_banco(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.iban:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_sol_nuevos_suministro(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.num_expediente_acometida and var.motivo_consulta == '02':
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_numero_expediente_fraude(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.num_expediente_fraude:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_cod_reclam_anterior(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.codigo_solicitud_reclamacion:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_importe_reclamado(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.importe_reclamado:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0

    def check_tipo_atencion_incorrecta(self):
        for var in self.r1.variables_detalle_reclamacion:
            if not var.tipo_de_atencion_incorrecta:
                return False
        return len(self.r1.variables_detalle_reclamacion) > 0


# Module Functions

def get_minimum_fields(r1_subtype):
    for x in SUBTYPES_R101:
        if x['code'] == r1_subtype:
            return x['min_fields']
    return []


def get_subtypes(r1_type):
    return [x['code'] for x in SUBTYPES_R101 if x['type'] == r1_type]


def get_type_from_subtype(r1_subtype):
    for x in SUBTYPES_R101:
        if x['code'] == r1_subtype:
            return x['type']
    return []



# -*- coding: utf-8 -*-
import unittest

from gestionatr.utils import get_description

from expects import *


class TestDefsTables(unittest.TestCase):
    def test_get_description(self):
        expect(
            get_description('02', 'TABLA_80')
        ).to(equal('Improcedente / Desfavorable'))

        def get_non_existent_code():
            return get_description('CODIGO_INVENTADO', 'TABLA_80')

        expect(get_non_existent_code).to(raise_error(ValueError))

        def get_non_existent_table():
            return get_description('01', 'TABLA_INVENTADA')

        expect(get_non_existent_table).to(raise_error(ValueError))



# -*- coding: utf-8 -*-
from gestionatr.output.messages.sw_c2 import *
from gestionatr.output.messages.sw_c1 import Medida
from gestionatr.output.messages.sw_f1 import DireccionSuministro


# Paso 01

class MensajeSolicitudTraspasoCOR(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'solicitud_traspaso_cor')

    def __init__(self):
        self.mensaje = XmlField('MensajeSolicitudTraspasoCOR',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.solicitud_traspaso_cor = SolicitudTraspasoCOR()
        super(MensajeSolicitudTraspasoCOR, self) \
            .__init__('MensajeSolicitudTraspasoCOR', 'mensaje')


class SolicitudTraspasoCOR(XmlModel):

    _sort_order = ('solicitud_traspaso_cor', 'datos_solicitud', 'contrato', 'cliente',
                   'direccion_ps', 'registros_documento')

    def __init__(self):
        self.solicitud_traspaso_cor = XmlField('SolicitudTraspasoCOR')
        self.datos_solicitud = DatosSolicitud()
        self.contrato = ContratoT101()
        self.cliente = Cliente()
        self.direccion_ps = DireccionPS()
        self.registros_documento = RegistrosDocumento()
        super(SolicitudTraspasoCOR, self).__init__('SolicitudTraspasoCOR', 'solicitud_traspaso_cor')


class DatosSolicitud(XmlModel):

    _sort_order = ('datos_solicitud', 'motivo_traspaso', 'fecha_prevista_accion', 'cnae',
                   'ind_esencial', 'fecha_ultimo_movimiento_ind_esencial', 'susp_baja_impago_en_curso')

    def __init__(self):
        self.datos_solicitud = XmlField('DatosSolicitud')
        self.motivo_traspaso = XmlField('MotivoTraspaso')
        self.fecha_prevista_accion = XmlField('FechaPrevistaAccion')
        self.cnae = XmlField('CNAE')
        self.ind_esencial = XmlField('IndEsencial')
        self.fecha_ultimo_movimiento_ind_esencial = XmlField('FechaUltimoMovimientoIndEsencial')
        self.susp_baja_impago_en_curso = XmlField('SuspBajaImpagoEnCurso')

        super(DatosSolicitud, self).__init__('DatosSolicitud', 'datos_solicitud')


class ContratoT101(Contrato):

    _sort_order = ('contrato', 'fecha_finalizacion', 'autoconsumo', 'tipo_contrato_atr',
                   'condiciones_contractuales', 'periodicidad_facturacion', 'consumo_anual_estimado', 'contacto')

    def __init__(self):
        super(ContratoT101, self).__init__()


class DireccionPS(DireccionSuministro):

    _sort_order = ('direccion', 'pais', 'provincia', 'municipio', 'poblacion', 'tipo_via', 'cod_postal',
                   'calle', 'numero_finca', 'duplicador_finca', 'escalera', 'piso', 'puerta',
                   'tipo_aclarador_finca', 'aclarador_finca')

    def __init__(self, name='DireccionPS'):
        super(DireccionPS, self).__init__(name=name)


# Paso 02

class MensajeAceptacionTraspasoCOR(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'aceptacion_traspaso_cor')

    def __init__(self):
        self.mensaje = XmlField('MensajeAceptacionTraspasoCOR',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.aceptacion_traspaso_cor = AceptacionTraspasoCOR()
        super(MensajeAceptacionTraspasoCOR, self).__init__('MensajeAceptacionTraspasoCOR', 'mensaje')


class MensajeRechazoTraspasoCOR(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'rechazos')

    def __init__(self):
        self.mensaje = XmlField('MensajeRechazoTraspasoCOR',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.rechazos = Rechazos()
        super(MensajeRechazoTraspasoCOR, self).__init__('MensajeRechazoTraspasoCOR', 'mensaje')


class AceptacionTraspasoCOR(XmlModel):

    _sort_order = ('aceptacion_traspaso_cor', 'datos_aceptacion')

    def __init__(self):
        self.aceptacion_traspaso_cor = XmlField('AceptacionTraspasoCOR')
        self.datos_aceptacion = DatosAceptacion()
        super(AceptacionTraspasoCOR, self).__init__('AceptacionTraspasoCOR', 'aceptacion_traspaso_cor')


class DatosAceptacion(DatosAceptacion):

    _sort_order = ('datos_aceptacion', 'fecha_aceptacion')

    def __init__(self):
        super(DatosAceptacion, self).__init__()


# Paso 05

class MensajeActivacionTraspasoCOR(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'activacion_traspaso_cor')

    def __init__(self):
        self.mensaje = XmlField('MensajeActivacionTraspasoCOR',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.activacion_traspaso_cor = ActivacionTraspasoCOR()
        super(MensajeActivacionTraspasoCOR, self).__init__('MensajeActivacionTraspasoCOR', 'mensaje')


class ActivacionTraspasoCOR(XmlModel):

    _sort_order = ('activacion_traspaso_cor', 'datos_activacion', 'contrato', 'puntos_de_medida')

    def __init__(self):
        self.activacion_traspaso_cor = XmlField('ActivacionTraspasoCOR')
        self.datos_activacion = DatosActivacion()
        self.contrato = Contrato()
        self.puntos_de_medida = PuntosDeMedida()
        super(ActivacionTraspasoCOR, self).__init__('ActivacionTraspasoCOR', 'activacion_traspaso_cor')


class DatosActivacion(XmlModel):

    _sort_order = ('datos_activacion', 'fecha_activacion', 'en_servicio', 'ind_esencial',
                   'fecha_ultimo_movimiento_ind_esencial')

    def __init__(self):
        self.datos_activacion = XmlField('DatosActivacion')
        self.fecha_activacion = XmlField('FechaActivacion')
        self.en_servicio = XmlField('EnServicio')
        self.ind_esencial = XmlField('IndEsencial')
        self.fecha_ultimo_movimiento_ind_esencial = XmlField('FechaUltimoMovimientoIndEsencial')
        super(DatosActivacion, self).__init__('DatosActivacion', 'datos_activacion')


# Paso 06

class MensajeActivacionTraspasoCORSaliente(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'notificacion_comercializador_saliente_t1')

    def __init__(self):
        self.mensaje = XmlField('MensajeActivacionComercializadorSalienteT1',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.notificacion_comercializador_saliente_t1 = NotificacionComercializadorSalienteT1()
        super(MensajeActivacionTraspasoCORSaliente, self).__init__('MensajeActivacionTraspasoCORSalienteT1', 'mensaje')


class NotificacionComercializadorSalienteT1(XmlModel):

    _sort_order = ('notificacion_comercializador_saliente_t1', 'datos_notificacion', 'contrato', 'puntos_de_medida')

    def __init__(self):
        self.notificacion_comercializador_saliente_t1 = XmlField('NotificacionComercializadorSalienteT1')
        self.datos_notificacion = DatosNotificacion()
        self.contrato = Contrato()
        self.puntos_de_medida = PuntosDeMedida()
        super(NotificacionComercializadorSalienteT1, self).__init__('NotificacionComercializadorSalienteT1', 'notificacion_comercializador_saliente_t1')


class Medida(Medida):

    _sort_order = ('medida', 'tipo_dhedm', 'periodo', 'magnitud_medida', 'procedencia', 'ultima_lectura_firme',
                   'fecha_lectura_firme', 'anomalia', 'comentarios')

    def __init__(self):
        super(Medida, self).__init__()






# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .message import Message, except_f1
from .C1 import C1
from .C2 import C2
from .A1 import A1
from .A3 import A3
from .B1 import B1
from .B2 import B2
from .M1 import M1
from .M2 import M2
from .D1 import D1
from .E1 import E1
from .E2 import E2
from .W1 import W1
from .P0 import P0
from .Q1 import Q1
from .R1 import R1
from .T1 import T1
from .F1 import F1
from .message_gas import MessageGas
from .A1_41 import A1_41
from .B70 import B7031, B7032, B70
from .A1_02 import A1_02
from .A1_05 import A1_05
from .A1_44 import A1_44
from .A1_03 import A1_03
from .A1_04 import A1_04
from .A1_48 import A1_48
from .A1_46 import A1_46
from .A1_49 import A1_49
from .A12_24 import A12_24
from .A12_26 import A12_26
from .A19_45 import A19_45
from .A1_38 import A1_38
from .A13_50 import A13_50
from .A20_36 import A20_36
from .A1_42 import A1_42
from .A1_43 import A1_43
from .A5_29 import A5_29
